# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['UserArgs', 'User']

@pulumi.input_type
class UserArgs:
    def __init__(__self__, *,
                 email: pulumi.Input[_builtins.str],
                 username: pulumi.Input[_builtins.str],
                 domain_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserDomainGrantArgs']]]] = None,
                 firewall_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserFirewallGrantArgs']]]] = None,
                 global_grants: Optional[pulumi.Input['UserGlobalGrantsArgs']] = None,
                 image_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserImageGrantArgs']]]] = None,
                 linode_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserLinodeGrantArgs']]]] = None,
                 longview_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserLongviewGrantArgs']]]] = None,
                 nodebalancer_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserNodebalancerGrantArgs']]]] = None,
                 restricted: Optional[pulumi.Input[_builtins.bool]] = None,
                 stackscript_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserStackscriptGrantArgs']]]] = None,
                 volume_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserVolumeGrantArgs']]]] = None,
                 vpc_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserVpcGrantArgs']]]] = None):
        """
        The set of arguments for constructing a User resource.
        :param pulumi.Input[_builtins.str] email: The email address of the user.
        :param pulumi.Input[_builtins.str] username: The username of the user.
        :param pulumi.Input[Sequence[pulumi.Input['UserDomainGrantArgs']]] domain_grants: The domains the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input['UserFirewallGrantArgs']]] firewall_grants: The firewalls the user has permissions access to.
        :param pulumi.Input['UserGlobalGrantsArgs'] global_grants: A structure containing the Account-level grants a User has.
        :param pulumi.Input[Sequence[pulumi.Input['UserImageGrantArgs']]] image_grants: The images the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input['UserLinodeGrantArgs']]] linode_grants: The Linodes the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input['UserLongviewGrantArgs']]] longview_grants: The longview the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input['UserNodebalancerGrantArgs']]] nodebalancer_grants: The NodeBalancers the user has permissions access to.
        :param pulumi.Input[_builtins.bool] restricted: If true, this user will only have explicit permissions granted.
               
               * `global_grants` - (optional) A structure containing the Account-level grants a User has.
               
               The following arguments are sets of entity grants:
        :param pulumi.Input[Sequence[pulumi.Input['UserStackscriptGrantArgs']]] stackscript_grants: The StackScripts the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input['UserVolumeGrantArgs']]] volume_grants: The volumes the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input['UserVpcGrantArgs']]] vpc_grants: The Virtual Private Clouds (VPCs) the user has permissions access to.
        """
        pulumi.set(__self__, "email", email)
        pulumi.set(__self__, "username", username)
        if domain_grants is not None:
            pulumi.set(__self__, "domain_grants", domain_grants)
        if firewall_grants is not None:
            pulumi.set(__self__, "firewall_grants", firewall_grants)
        if global_grants is not None:
            pulumi.set(__self__, "global_grants", global_grants)
        if image_grants is not None:
            pulumi.set(__self__, "image_grants", image_grants)
        if linode_grants is not None:
            pulumi.set(__self__, "linode_grants", linode_grants)
        if longview_grants is not None:
            pulumi.set(__self__, "longview_grants", longview_grants)
        if nodebalancer_grants is not None:
            pulumi.set(__self__, "nodebalancer_grants", nodebalancer_grants)
        if restricted is not None:
            pulumi.set(__self__, "restricted", restricted)
        if stackscript_grants is not None:
            pulumi.set(__self__, "stackscript_grants", stackscript_grants)
        if volume_grants is not None:
            pulumi.set(__self__, "volume_grants", volume_grants)
        if vpc_grants is not None:
            pulumi.set(__self__, "vpc_grants", vpc_grants)

    @_builtins.property
    @pulumi.getter
    def email(self) -> pulumi.Input[_builtins.str]:
        """
        The email address of the user.
        """
        return pulumi.get(self, "email")

    @email.setter
    def email(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "email", value)

    @_builtins.property
    @pulumi.getter
    def username(self) -> pulumi.Input[_builtins.str]:
        """
        The username of the user.
        """
        return pulumi.get(self, "username")

    @username.setter
    def username(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "username", value)

    @_builtins.property
    @pulumi.getter(name="domainGrants")
    def domain_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserDomainGrantArgs']]]]:
        """
        The domains the user has permissions access to.
        """
        return pulumi.get(self, "domain_grants")

    @domain_grants.setter
    def domain_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserDomainGrantArgs']]]]):
        pulumi.set(self, "domain_grants", value)

    @_builtins.property
    @pulumi.getter(name="firewallGrants")
    def firewall_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserFirewallGrantArgs']]]]:
        """
        The firewalls the user has permissions access to.
        """
        return pulumi.get(self, "firewall_grants")

    @firewall_grants.setter
    def firewall_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserFirewallGrantArgs']]]]):
        pulumi.set(self, "firewall_grants", value)

    @_builtins.property
    @pulumi.getter(name="globalGrants")
    def global_grants(self) -> Optional[pulumi.Input['UserGlobalGrantsArgs']]:
        """
        A structure containing the Account-level grants a User has.
        """
        return pulumi.get(self, "global_grants")

    @global_grants.setter
    def global_grants(self, value: Optional[pulumi.Input['UserGlobalGrantsArgs']]):
        pulumi.set(self, "global_grants", value)

    @_builtins.property
    @pulumi.getter(name="imageGrants")
    def image_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserImageGrantArgs']]]]:
        """
        The images the user has permissions access to.
        """
        return pulumi.get(self, "image_grants")

    @image_grants.setter
    def image_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserImageGrantArgs']]]]):
        pulumi.set(self, "image_grants", value)

    @_builtins.property
    @pulumi.getter(name="linodeGrants")
    def linode_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserLinodeGrantArgs']]]]:
        """
        The Linodes the user has permissions access to.
        """
        return pulumi.get(self, "linode_grants")

    @linode_grants.setter
    def linode_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserLinodeGrantArgs']]]]):
        pulumi.set(self, "linode_grants", value)

    @_builtins.property
    @pulumi.getter(name="longviewGrants")
    def longview_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserLongviewGrantArgs']]]]:
        """
        The longview the user has permissions access to.
        """
        return pulumi.get(self, "longview_grants")

    @longview_grants.setter
    def longview_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserLongviewGrantArgs']]]]):
        pulumi.set(self, "longview_grants", value)

    @_builtins.property
    @pulumi.getter(name="nodebalancerGrants")
    def nodebalancer_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserNodebalancerGrantArgs']]]]:
        """
        The NodeBalancers the user has permissions access to.
        """
        return pulumi.get(self, "nodebalancer_grants")

    @nodebalancer_grants.setter
    def nodebalancer_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserNodebalancerGrantArgs']]]]):
        pulumi.set(self, "nodebalancer_grants", value)

    @_builtins.property
    @pulumi.getter
    def restricted(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        If true, this user will only have explicit permissions granted.

        * `global_grants` - (optional) A structure containing the Account-level grants a User has.

        The following arguments are sets of entity grants:
        """
        return pulumi.get(self, "restricted")

    @restricted.setter
    def restricted(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "restricted", value)

    @_builtins.property
    @pulumi.getter(name="stackscriptGrants")
    def stackscript_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserStackscriptGrantArgs']]]]:
        """
        The StackScripts the user has permissions access to.
        """
        return pulumi.get(self, "stackscript_grants")

    @stackscript_grants.setter
    def stackscript_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserStackscriptGrantArgs']]]]):
        pulumi.set(self, "stackscript_grants", value)

    @_builtins.property
    @pulumi.getter(name="volumeGrants")
    def volume_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserVolumeGrantArgs']]]]:
        """
        The volumes the user has permissions access to.
        """
        return pulumi.get(self, "volume_grants")

    @volume_grants.setter
    def volume_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserVolumeGrantArgs']]]]):
        pulumi.set(self, "volume_grants", value)

    @_builtins.property
    @pulumi.getter(name="vpcGrants")
    def vpc_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserVpcGrantArgs']]]]:
        """
        The Virtual Private Clouds (VPCs) the user has permissions access to.
        """
        return pulumi.get(self, "vpc_grants")

    @vpc_grants.setter
    def vpc_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserVpcGrantArgs']]]]):
        pulumi.set(self, "vpc_grants", value)


@pulumi.input_type
class _UserState:
    def __init__(__self__, *,
                 domain_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserDomainGrantArgs']]]] = None,
                 email: Optional[pulumi.Input[_builtins.str]] = None,
                 firewall_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserFirewallGrantArgs']]]] = None,
                 global_grants: Optional[pulumi.Input['UserGlobalGrantsArgs']] = None,
                 image_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserImageGrantArgs']]]] = None,
                 linode_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserLinodeGrantArgs']]]] = None,
                 longview_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserLongviewGrantArgs']]]] = None,
                 nodebalancer_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserNodebalancerGrantArgs']]]] = None,
                 restricted: Optional[pulumi.Input[_builtins.bool]] = None,
                 ssh_keys: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 stackscript_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserStackscriptGrantArgs']]]] = None,
                 tfa_enabled: Optional[pulumi.Input[_builtins.bool]] = None,
                 user_type: Optional[pulumi.Input[_builtins.str]] = None,
                 username: Optional[pulumi.Input[_builtins.str]] = None,
                 volume_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserVolumeGrantArgs']]]] = None,
                 vpc_grants: Optional[pulumi.Input[Sequence[pulumi.Input['UserVpcGrantArgs']]]] = None):
        """
        Input properties used for looking up and filtering User resources.
        :param pulumi.Input[Sequence[pulumi.Input['UserDomainGrantArgs']]] domain_grants: The domains the user has permissions access to.
        :param pulumi.Input[_builtins.str] email: The email address of the user.
        :param pulumi.Input[Sequence[pulumi.Input['UserFirewallGrantArgs']]] firewall_grants: The firewalls the user has permissions access to.
        :param pulumi.Input['UserGlobalGrantsArgs'] global_grants: A structure containing the Account-level grants a User has.
        :param pulumi.Input[Sequence[pulumi.Input['UserImageGrantArgs']]] image_grants: The images the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input['UserLinodeGrantArgs']]] linode_grants: The Linodes the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input['UserLongviewGrantArgs']]] longview_grants: The longview the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input['UserNodebalancerGrantArgs']]] nodebalancer_grants: The NodeBalancers the user has permissions access to.
        :param pulumi.Input[_builtins.bool] restricted: If true, this user will only have explicit permissions granted.
               
               * `global_grants` - (optional) A structure containing the Account-level grants a User has.
               
               The following arguments are sets of entity grants:
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] ssh_keys: A list of the User's SSH keys.
        :param pulumi.Input[Sequence[pulumi.Input['UserStackscriptGrantArgs']]] stackscript_grants: The StackScripts the user has permissions access to.
        :param pulumi.Input[_builtins.bool] tfa_enabled: Whether the user has two-factor-authentication enabled.
        :param pulumi.Input[_builtins.str] user_type: The type of this user.
        :param pulumi.Input[_builtins.str] username: The username of the user.
        :param pulumi.Input[Sequence[pulumi.Input['UserVolumeGrantArgs']]] volume_grants: The volumes the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input['UserVpcGrantArgs']]] vpc_grants: The Virtual Private Clouds (VPCs) the user has permissions access to.
        """
        if domain_grants is not None:
            pulumi.set(__self__, "domain_grants", domain_grants)
        if email is not None:
            pulumi.set(__self__, "email", email)
        if firewall_grants is not None:
            pulumi.set(__self__, "firewall_grants", firewall_grants)
        if global_grants is not None:
            pulumi.set(__self__, "global_grants", global_grants)
        if image_grants is not None:
            pulumi.set(__self__, "image_grants", image_grants)
        if linode_grants is not None:
            pulumi.set(__self__, "linode_grants", linode_grants)
        if longview_grants is not None:
            pulumi.set(__self__, "longview_grants", longview_grants)
        if nodebalancer_grants is not None:
            pulumi.set(__self__, "nodebalancer_grants", nodebalancer_grants)
        if restricted is not None:
            pulumi.set(__self__, "restricted", restricted)
        if ssh_keys is not None:
            pulumi.set(__self__, "ssh_keys", ssh_keys)
        if stackscript_grants is not None:
            pulumi.set(__self__, "stackscript_grants", stackscript_grants)
        if tfa_enabled is not None:
            pulumi.set(__self__, "tfa_enabled", tfa_enabled)
        if user_type is not None:
            pulumi.set(__self__, "user_type", user_type)
        if username is not None:
            pulumi.set(__self__, "username", username)
        if volume_grants is not None:
            pulumi.set(__self__, "volume_grants", volume_grants)
        if vpc_grants is not None:
            pulumi.set(__self__, "vpc_grants", vpc_grants)

    @_builtins.property
    @pulumi.getter(name="domainGrants")
    def domain_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserDomainGrantArgs']]]]:
        """
        The domains the user has permissions access to.
        """
        return pulumi.get(self, "domain_grants")

    @domain_grants.setter
    def domain_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserDomainGrantArgs']]]]):
        pulumi.set(self, "domain_grants", value)

    @_builtins.property
    @pulumi.getter
    def email(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The email address of the user.
        """
        return pulumi.get(self, "email")

    @email.setter
    def email(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "email", value)

    @_builtins.property
    @pulumi.getter(name="firewallGrants")
    def firewall_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserFirewallGrantArgs']]]]:
        """
        The firewalls the user has permissions access to.
        """
        return pulumi.get(self, "firewall_grants")

    @firewall_grants.setter
    def firewall_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserFirewallGrantArgs']]]]):
        pulumi.set(self, "firewall_grants", value)

    @_builtins.property
    @pulumi.getter(name="globalGrants")
    def global_grants(self) -> Optional[pulumi.Input['UserGlobalGrantsArgs']]:
        """
        A structure containing the Account-level grants a User has.
        """
        return pulumi.get(self, "global_grants")

    @global_grants.setter
    def global_grants(self, value: Optional[pulumi.Input['UserGlobalGrantsArgs']]):
        pulumi.set(self, "global_grants", value)

    @_builtins.property
    @pulumi.getter(name="imageGrants")
    def image_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserImageGrantArgs']]]]:
        """
        The images the user has permissions access to.
        """
        return pulumi.get(self, "image_grants")

    @image_grants.setter
    def image_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserImageGrantArgs']]]]):
        pulumi.set(self, "image_grants", value)

    @_builtins.property
    @pulumi.getter(name="linodeGrants")
    def linode_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserLinodeGrantArgs']]]]:
        """
        The Linodes the user has permissions access to.
        """
        return pulumi.get(self, "linode_grants")

    @linode_grants.setter
    def linode_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserLinodeGrantArgs']]]]):
        pulumi.set(self, "linode_grants", value)

    @_builtins.property
    @pulumi.getter(name="longviewGrants")
    def longview_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserLongviewGrantArgs']]]]:
        """
        The longview the user has permissions access to.
        """
        return pulumi.get(self, "longview_grants")

    @longview_grants.setter
    def longview_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserLongviewGrantArgs']]]]):
        pulumi.set(self, "longview_grants", value)

    @_builtins.property
    @pulumi.getter(name="nodebalancerGrants")
    def nodebalancer_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserNodebalancerGrantArgs']]]]:
        """
        The NodeBalancers the user has permissions access to.
        """
        return pulumi.get(self, "nodebalancer_grants")

    @nodebalancer_grants.setter
    def nodebalancer_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserNodebalancerGrantArgs']]]]):
        pulumi.set(self, "nodebalancer_grants", value)

    @_builtins.property
    @pulumi.getter
    def restricted(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        If true, this user will only have explicit permissions granted.

        * `global_grants` - (optional) A structure containing the Account-level grants a User has.

        The following arguments are sets of entity grants:
        """
        return pulumi.get(self, "restricted")

    @restricted.setter
    def restricted(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "restricted", value)

    @_builtins.property
    @pulumi.getter(name="sshKeys")
    def ssh_keys(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A list of the User's SSH keys.
        """
        return pulumi.get(self, "ssh_keys")

    @ssh_keys.setter
    def ssh_keys(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "ssh_keys", value)

    @_builtins.property
    @pulumi.getter(name="stackscriptGrants")
    def stackscript_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserStackscriptGrantArgs']]]]:
        """
        The StackScripts the user has permissions access to.
        """
        return pulumi.get(self, "stackscript_grants")

    @stackscript_grants.setter
    def stackscript_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserStackscriptGrantArgs']]]]):
        pulumi.set(self, "stackscript_grants", value)

    @_builtins.property
    @pulumi.getter(name="tfaEnabled")
    def tfa_enabled(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Whether the user has two-factor-authentication enabled.
        """
        return pulumi.get(self, "tfa_enabled")

    @tfa_enabled.setter
    def tfa_enabled(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "tfa_enabled", value)

    @_builtins.property
    @pulumi.getter(name="userType")
    def user_type(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The type of this user.
        """
        return pulumi.get(self, "user_type")

    @user_type.setter
    def user_type(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "user_type", value)

    @_builtins.property
    @pulumi.getter
    def username(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The username of the user.
        """
        return pulumi.get(self, "username")

    @username.setter
    def username(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "username", value)

    @_builtins.property
    @pulumi.getter(name="volumeGrants")
    def volume_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserVolumeGrantArgs']]]]:
        """
        The volumes the user has permissions access to.
        """
        return pulumi.get(self, "volume_grants")

    @volume_grants.setter
    def volume_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserVolumeGrantArgs']]]]):
        pulumi.set(self, "volume_grants", value)

    @_builtins.property
    @pulumi.getter(name="vpcGrants")
    def vpc_grants(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserVpcGrantArgs']]]]:
        """
        The Virtual Private Clouds (VPCs) the user has permissions access to.
        """
        return pulumi.get(self, "vpc_grants")

    @vpc_grants.setter
    def vpc_grants(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserVpcGrantArgs']]]]):
        pulumi.set(self, "vpc_grants", value)


@pulumi.type_token("linode:index/user:User")
class User(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 domain_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserDomainGrantArgs', 'UserDomainGrantArgsDict']]]]] = None,
                 email: Optional[pulumi.Input[_builtins.str]] = None,
                 firewall_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserFirewallGrantArgs', 'UserFirewallGrantArgsDict']]]]] = None,
                 global_grants: Optional[pulumi.Input[Union['UserGlobalGrantsArgs', 'UserGlobalGrantsArgsDict']]] = None,
                 image_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserImageGrantArgs', 'UserImageGrantArgsDict']]]]] = None,
                 linode_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserLinodeGrantArgs', 'UserLinodeGrantArgsDict']]]]] = None,
                 longview_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserLongviewGrantArgs', 'UserLongviewGrantArgsDict']]]]] = None,
                 nodebalancer_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserNodebalancerGrantArgs', 'UserNodebalancerGrantArgsDict']]]]] = None,
                 restricted: Optional[pulumi.Input[_builtins.bool]] = None,
                 stackscript_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserStackscriptGrantArgs', 'UserStackscriptGrantArgsDict']]]]] = None,
                 username: Optional[pulumi.Input[_builtins.str]] = None,
                 volume_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserVolumeGrantArgs', 'UserVolumeGrantArgsDict']]]]] = None,
                 vpc_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserVpcGrantArgs', 'UserVpcGrantArgsDict']]]]] = None,
                 __props__=None):
        """
        Manages a Linode User.
        For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/post-user).

        ## Example Usage

        Create an unrestricted user:

        ```python
        import pulumi
        import pulumi_linode as linode

        john = linode.User("john",
            username="john123",
            email="john@acme.io")
        ```

        Create a restricted user with grants:

        ```python
        import pulumi
        import pulumi_linode as linode

        fooser = linode.User("fooser",
            username="cooluser123",
            email="cool@acme.io",
            restricted=True,
            global_grants={
                "add_linodes": True,
                "add_images": True,
            },
            linode_grants=[{
                "id": 12345,
                "permissions": "read_write",
            }])
        ```

        ## Global Grants

        * `account_access` - (optional) The level of access this User has to Account-level actions, like billing information. (`read_only`, `read_write`)

        * `add_domains` - (optional) If true, this User may add Domains.

        * `add_databases` - (optional) If true, this User may add Databases.

        * `add_firewalls` - (optional) If true, this User may add Firewalls.

        * `add_images` - (optional) If true, this User may add Images.

        * `add_linodes` - (optional) If true, this User may create Linodes.

        * `add_longview` - (optional) If true, this User may create Longview clients and view the current plan.

        * `add_nodebalancers` - (optional) If true, this User may add NodeBalancers.

        * `add_stackscripts` - (optional) If true, this User may add StackScripts.

        * `add_vpcs` - (optional) If true, this User may add Virtual Private Clouds (VPCs).

        * `cancel_account` - (optional) If true, this User may cancel the entire Account.

        * `longview_subscription` - (optional) If true, this User may manage the Account’s Longview subscription.

        ## Entity Grants

        * `id` - (required) The ID of the entity this grant applies to.

        * `permissions` - (required) The level of access this User has to this entity. (`read_only`, `read_write`)

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserDomainGrantArgs', 'UserDomainGrantArgsDict']]]] domain_grants: The domains the user has permissions access to.
        :param pulumi.Input[_builtins.str] email: The email address of the user.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserFirewallGrantArgs', 'UserFirewallGrantArgsDict']]]] firewall_grants: The firewalls the user has permissions access to.
        :param pulumi.Input[Union['UserGlobalGrantsArgs', 'UserGlobalGrantsArgsDict']] global_grants: A structure containing the Account-level grants a User has.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserImageGrantArgs', 'UserImageGrantArgsDict']]]] image_grants: The images the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserLinodeGrantArgs', 'UserLinodeGrantArgsDict']]]] linode_grants: The Linodes the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserLongviewGrantArgs', 'UserLongviewGrantArgsDict']]]] longview_grants: The longview the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserNodebalancerGrantArgs', 'UserNodebalancerGrantArgsDict']]]] nodebalancer_grants: The NodeBalancers the user has permissions access to.
        :param pulumi.Input[_builtins.bool] restricted: If true, this user will only have explicit permissions granted.
               
               * `global_grants` - (optional) A structure containing the Account-level grants a User has.
               
               The following arguments are sets of entity grants:
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserStackscriptGrantArgs', 'UserStackscriptGrantArgsDict']]]] stackscript_grants: The StackScripts the user has permissions access to.
        :param pulumi.Input[_builtins.str] username: The username of the user.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserVolumeGrantArgs', 'UserVolumeGrantArgsDict']]]] volume_grants: The volumes the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserVpcGrantArgs', 'UserVpcGrantArgsDict']]]] vpc_grants: The Virtual Private Clouds (VPCs) the user has permissions access to.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: UserArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a Linode User.
        For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/post-user).

        ## Example Usage

        Create an unrestricted user:

        ```python
        import pulumi
        import pulumi_linode as linode

        john = linode.User("john",
            username="john123",
            email="john@acme.io")
        ```

        Create a restricted user with grants:

        ```python
        import pulumi
        import pulumi_linode as linode

        fooser = linode.User("fooser",
            username="cooluser123",
            email="cool@acme.io",
            restricted=True,
            global_grants={
                "add_linodes": True,
                "add_images": True,
            },
            linode_grants=[{
                "id": 12345,
                "permissions": "read_write",
            }])
        ```

        ## Global Grants

        * `account_access` - (optional) The level of access this User has to Account-level actions, like billing information. (`read_only`, `read_write`)

        * `add_domains` - (optional) If true, this User may add Domains.

        * `add_databases` - (optional) If true, this User may add Databases.

        * `add_firewalls` - (optional) If true, this User may add Firewalls.

        * `add_images` - (optional) If true, this User may add Images.

        * `add_linodes` - (optional) If true, this User may create Linodes.

        * `add_longview` - (optional) If true, this User may create Longview clients and view the current plan.

        * `add_nodebalancers` - (optional) If true, this User may add NodeBalancers.

        * `add_stackscripts` - (optional) If true, this User may add StackScripts.

        * `add_vpcs` - (optional) If true, this User may add Virtual Private Clouds (VPCs).

        * `cancel_account` - (optional) If true, this User may cancel the entire Account.

        * `longview_subscription` - (optional) If true, this User may manage the Account’s Longview subscription.

        ## Entity Grants

        * `id` - (required) The ID of the entity this grant applies to.

        * `permissions` - (required) The level of access this User has to this entity. (`read_only`, `read_write`)

        :param str resource_name: The name of the resource.
        :param UserArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(UserArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 domain_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserDomainGrantArgs', 'UserDomainGrantArgsDict']]]]] = None,
                 email: Optional[pulumi.Input[_builtins.str]] = None,
                 firewall_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserFirewallGrantArgs', 'UserFirewallGrantArgsDict']]]]] = None,
                 global_grants: Optional[pulumi.Input[Union['UserGlobalGrantsArgs', 'UserGlobalGrantsArgsDict']]] = None,
                 image_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserImageGrantArgs', 'UserImageGrantArgsDict']]]]] = None,
                 linode_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserLinodeGrantArgs', 'UserLinodeGrantArgsDict']]]]] = None,
                 longview_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserLongviewGrantArgs', 'UserLongviewGrantArgsDict']]]]] = None,
                 nodebalancer_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserNodebalancerGrantArgs', 'UserNodebalancerGrantArgsDict']]]]] = None,
                 restricted: Optional[pulumi.Input[_builtins.bool]] = None,
                 stackscript_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserStackscriptGrantArgs', 'UserStackscriptGrantArgsDict']]]]] = None,
                 username: Optional[pulumi.Input[_builtins.str]] = None,
                 volume_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserVolumeGrantArgs', 'UserVolumeGrantArgsDict']]]]] = None,
                 vpc_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserVpcGrantArgs', 'UserVpcGrantArgsDict']]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = UserArgs.__new__(UserArgs)

            __props__.__dict__["domain_grants"] = domain_grants
            if email is None and not opts.urn:
                raise TypeError("Missing required property 'email'")
            __props__.__dict__["email"] = email
            __props__.__dict__["firewall_grants"] = firewall_grants
            __props__.__dict__["global_grants"] = global_grants
            __props__.__dict__["image_grants"] = image_grants
            __props__.__dict__["linode_grants"] = linode_grants
            __props__.__dict__["longview_grants"] = longview_grants
            __props__.__dict__["nodebalancer_grants"] = nodebalancer_grants
            __props__.__dict__["restricted"] = restricted
            __props__.__dict__["stackscript_grants"] = stackscript_grants
            if username is None and not opts.urn:
                raise TypeError("Missing required property 'username'")
            __props__.__dict__["username"] = username
            __props__.__dict__["volume_grants"] = volume_grants
            __props__.__dict__["vpc_grants"] = vpc_grants
            __props__.__dict__["ssh_keys"] = None
            __props__.__dict__["tfa_enabled"] = None
            __props__.__dict__["user_type"] = None
        super(User, __self__).__init__(
            'linode:index/user:User',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            domain_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserDomainGrantArgs', 'UserDomainGrantArgsDict']]]]] = None,
            email: Optional[pulumi.Input[_builtins.str]] = None,
            firewall_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserFirewallGrantArgs', 'UserFirewallGrantArgsDict']]]]] = None,
            global_grants: Optional[pulumi.Input[Union['UserGlobalGrantsArgs', 'UserGlobalGrantsArgsDict']]] = None,
            image_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserImageGrantArgs', 'UserImageGrantArgsDict']]]]] = None,
            linode_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserLinodeGrantArgs', 'UserLinodeGrantArgsDict']]]]] = None,
            longview_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserLongviewGrantArgs', 'UserLongviewGrantArgsDict']]]]] = None,
            nodebalancer_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserNodebalancerGrantArgs', 'UserNodebalancerGrantArgsDict']]]]] = None,
            restricted: Optional[pulumi.Input[_builtins.bool]] = None,
            ssh_keys: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            stackscript_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserStackscriptGrantArgs', 'UserStackscriptGrantArgsDict']]]]] = None,
            tfa_enabled: Optional[pulumi.Input[_builtins.bool]] = None,
            user_type: Optional[pulumi.Input[_builtins.str]] = None,
            username: Optional[pulumi.Input[_builtins.str]] = None,
            volume_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserVolumeGrantArgs', 'UserVolumeGrantArgsDict']]]]] = None,
            vpc_grants: Optional[pulumi.Input[Sequence[pulumi.Input[Union['UserVpcGrantArgs', 'UserVpcGrantArgsDict']]]]] = None) -> 'User':
        """
        Get an existing User resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserDomainGrantArgs', 'UserDomainGrantArgsDict']]]] domain_grants: The domains the user has permissions access to.
        :param pulumi.Input[_builtins.str] email: The email address of the user.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserFirewallGrantArgs', 'UserFirewallGrantArgsDict']]]] firewall_grants: The firewalls the user has permissions access to.
        :param pulumi.Input[Union['UserGlobalGrantsArgs', 'UserGlobalGrantsArgsDict']] global_grants: A structure containing the Account-level grants a User has.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserImageGrantArgs', 'UserImageGrantArgsDict']]]] image_grants: The images the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserLinodeGrantArgs', 'UserLinodeGrantArgsDict']]]] linode_grants: The Linodes the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserLongviewGrantArgs', 'UserLongviewGrantArgsDict']]]] longview_grants: The longview the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserNodebalancerGrantArgs', 'UserNodebalancerGrantArgsDict']]]] nodebalancer_grants: The NodeBalancers the user has permissions access to.
        :param pulumi.Input[_builtins.bool] restricted: If true, this user will only have explicit permissions granted.
               
               * `global_grants` - (optional) A structure containing the Account-level grants a User has.
               
               The following arguments are sets of entity grants:
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] ssh_keys: A list of the User's SSH keys.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserStackscriptGrantArgs', 'UserStackscriptGrantArgsDict']]]] stackscript_grants: The StackScripts the user has permissions access to.
        :param pulumi.Input[_builtins.bool] tfa_enabled: Whether the user has two-factor-authentication enabled.
        :param pulumi.Input[_builtins.str] user_type: The type of this user.
        :param pulumi.Input[_builtins.str] username: The username of the user.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserVolumeGrantArgs', 'UserVolumeGrantArgsDict']]]] volume_grants: The volumes the user has permissions access to.
        :param pulumi.Input[Sequence[pulumi.Input[Union['UserVpcGrantArgs', 'UserVpcGrantArgsDict']]]] vpc_grants: The Virtual Private Clouds (VPCs) the user has permissions access to.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _UserState.__new__(_UserState)

        __props__.__dict__["domain_grants"] = domain_grants
        __props__.__dict__["email"] = email
        __props__.__dict__["firewall_grants"] = firewall_grants
        __props__.__dict__["global_grants"] = global_grants
        __props__.__dict__["image_grants"] = image_grants
        __props__.__dict__["linode_grants"] = linode_grants
        __props__.__dict__["longview_grants"] = longview_grants
        __props__.__dict__["nodebalancer_grants"] = nodebalancer_grants
        __props__.__dict__["restricted"] = restricted
        __props__.__dict__["ssh_keys"] = ssh_keys
        __props__.__dict__["stackscript_grants"] = stackscript_grants
        __props__.__dict__["tfa_enabled"] = tfa_enabled
        __props__.__dict__["user_type"] = user_type
        __props__.__dict__["username"] = username
        __props__.__dict__["volume_grants"] = volume_grants
        __props__.__dict__["vpc_grants"] = vpc_grants
        return User(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="domainGrants")
    def domain_grants(self) -> pulumi.Output[Sequence['outputs.UserDomainGrant']]:
        """
        The domains the user has permissions access to.
        """
        return pulumi.get(self, "domain_grants")

    @_builtins.property
    @pulumi.getter
    def email(self) -> pulumi.Output[_builtins.str]:
        """
        The email address of the user.
        """
        return pulumi.get(self, "email")

    @_builtins.property
    @pulumi.getter(name="firewallGrants")
    def firewall_grants(self) -> pulumi.Output[Sequence['outputs.UserFirewallGrant']]:
        """
        The firewalls the user has permissions access to.
        """
        return pulumi.get(self, "firewall_grants")

    @_builtins.property
    @pulumi.getter(name="globalGrants")
    def global_grants(self) -> pulumi.Output['outputs.UserGlobalGrants']:
        """
        A structure containing the Account-level grants a User has.
        """
        return pulumi.get(self, "global_grants")

    @_builtins.property
    @pulumi.getter(name="imageGrants")
    def image_grants(self) -> pulumi.Output[Sequence['outputs.UserImageGrant']]:
        """
        The images the user has permissions access to.
        """
        return pulumi.get(self, "image_grants")

    @_builtins.property
    @pulumi.getter(name="linodeGrants")
    def linode_grants(self) -> pulumi.Output[Sequence['outputs.UserLinodeGrant']]:
        """
        The Linodes the user has permissions access to.
        """
        return pulumi.get(self, "linode_grants")

    @_builtins.property
    @pulumi.getter(name="longviewGrants")
    def longview_grants(self) -> pulumi.Output[Sequence['outputs.UserLongviewGrant']]:
        """
        The longview the user has permissions access to.
        """
        return pulumi.get(self, "longview_grants")

    @_builtins.property
    @pulumi.getter(name="nodebalancerGrants")
    def nodebalancer_grants(self) -> pulumi.Output[Sequence['outputs.UserNodebalancerGrant']]:
        """
        The NodeBalancers the user has permissions access to.
        """
        return pulumi.get(self, "nodebalancer_grants")

    @_builtins.property
    @pulumi.getter
    def restricted(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        If true, this user will only have explicit permissions granted.

        * `global_grants` - (optional) A structure containing the Account-level grants a User has.

        The following arguments are sets of entity grants:
        """
        return pulumi.get(self, "restricted")

    @_builtins.property
    @pulumi.getter(name="sshKeys")
    def ssh_keys(self) -> pulumi.Output[Sequence[_builtins.str]]:
        """
        A list of the User's SSH keys.
        """
        return pulumi.get(self, "ssh_keys")

    @_builtins.property
    @pulumi.getter(name="stackscriptGrants")
    def stackscript_grants(self) -> pulumi.Output[Sequence['outputs.UserStackscriptGrant']]:
        """
        The StackScripts the user has permissions access to.
        """
        return pulumi.get(self, "stackscript_grants")

    @_builtins.property
    @pulumi.getter(name="tfaEnabled")
    def tfa_enabled(self) -> pulumi.Output[_builtins.bool]:
        """
        Whether the user has two-factor-authentication enabled.
        """
        return pulumi.get(self, "tfa_enabled")

    @_builtins.property
    @pulumi.getter(name="userType")
    def user_type(self) -> pulumi.Output[_builtins.str]:
        """
        The type of this user.
        """
        return pulumi.get(self, "user_type")

    @_builtins.property
    @pulumi.getter
    def username(self) -> pulumi.Output[_builtins.str]:
        """
        The username of the user.
        """
        return pulumi.get(self, "username")

    @_builtins.property
    @pulumi.getter(name="volumeGrants")
    def volume_grants(self) -> pulumi.Output[Sequence['outputs.UserVolumeGrant']]:
        """
        The volumes the user has permissions access to.
        """
        return pulumi.get(self, "volume_grants")

    @_builtins.property
    @pulumi.getter(name="vpcGrants")
    def vpc_grants(self) -> pulumi.Output[Sequence['outputs.UserVpcGrant']]:
        """
        The Virtual Private Clouds (VPCs) the user has permissions access to.
        """
        return pulumi.get(self, "vpc_grants")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetNodeBalancerResult',
    'AwaitableGetNodeBalancerResult',
    'get_node_balancer',
    'get_node_balancer_output',
]

@pulumi.output_type
class GetNodeBalancerResult:
    """
    A collection of values returned by getNodeBalancer.
    """
    def __init__(__self__, client_conn_throttle=None, client_udp_sess_throttle=None, created=None, firewalls=None, hostname=None, id=None, ipv4=None, ipv6=None, label=None, region=None, tags=None, transfers=None, updated=None, vpcs=None):
        if client_conn_throttle and not isinstance(client_conn_throttle, int):
            raise TypeError("Expected argument 'client_conn_throttle' to be a int")
        pulumi.set(__self__, "client_conn_throttle", client_conn_throttle)
        if client_udp_sess_throttle and not isinstance(client_udp_sess_throttle, int):
            raise TypeError("Expected argument 'client_udp_sess_throttle' to be a int")
        pulumi.set(__self__, "client_udp_sess_throttle", client_udp_sess_throttle)
        if created and not isinstance(created, str):
            raise TypeError("Expected argument 'created' to be a str")
        pulumi.set(__self__, "created", created)
        if firewalls and not isinstance(firewalls, list):
            raise TypeError("Expected argument 'firewalls' to be a list")
        pulumi.set(__self__, "firewalls", firewalls)
        if hostname and not isinstance(hostname, str):
            raise TypeError("Expected argument 'hostname' to be a str")
        pulumi.set(__self__, "hostname", hostname)
        if id and not isinstance(id, int):
            raise TypeError("Expected argument 'id' to be a int")
        pulumi.set(__self__, "id", id)
        if ipv4 and not isinstance(ipv4, str):
            raise TypeError("Expected argument 'ipv4' to be a str")
        pulumi.set(__self__, "ipv4", ipv4)
        if ipv6 and not isinstance(ipv6, str):
            raise TypeError("Expected argument 'ipv6' to be a str")
        pulumi.set(__self__, "ipv6", ipv6)
        if label and not isinstance(label, str):
            raise TypeError("Expected argument 'label' to be a str")
        pulumi.set(__self__, "label", label)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if tags and not isinstance(tags, list):
            raise TypeError("Expected argument 'tags' to be a list")
        pulumi.set(__self__, "tags", tags)
        if transfers and not isinstance(transfers, list):
            raise TypeError("Expected argument 'transfers' to be a list")
        pulumi.set(__self__, "transfers", transfers)
        if updated and not isinstance(updated, str):
            raise TypeError("Expected argument 'updated' to be a str")
        pulumi.set(__self__, "updated", updated)
        if vpcs and not isinstance(vpcs, list):
            raise TypeError("Expected argument 'vpcs' to be a list")
        pulumi.set(__self__, "vpcs", vpcs)

    @_builtins.property
    @pulumi.getter(name="clientConnThrottle")
    def client_conn_throttle(self) -> _builtins.int:
        """
        Throttle connections per second (0-20).
        """
        return pulumi.get(self, "client_conn_throttle")

    @_builtins.property
    @pulumi.getter(name="clientUdpSessThrottle")
    def client_udp_sess_throttle(self) -> _builtins.int:
        """
        Throttle UDP sessions per second (0-20).
        """
        return pulumi.get(self, "client_udp_sess_throttle")

    @_builtins.property
    @pulumi.getter
    def created(self) -> _builtins.str:
        """
        When this firewall was created.
        """
        return pulumi.get(self, "created")

    @_builtins.property
    @pulumi.getter
    def firewalls(self) -> Optional[Sequence['outputs.GetNodeBalancerFirewallResult']]:
        return pulumi.get(self, "firewalls")

    @_builtins.property
    @pulumi.getter
    def hostname(self) -> _builtins.str:
        """
        This NodeBalancer's hostname, ending with .ip.linodeusercontent.com
        """
        return pulumi.get(self, "hostname")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.int:
        """
        The Firewall's ID.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def ipv4(self) -> _builtins.str:
        """
        A list of IPv4 addresses or networks. Must be in IP/mask format.
        """
        return pulumi.get(self, "ipv4")

    @_builtins.property
    @pulumi.getter
    def ipv6(self) -> _builtins.str:
        """
        A list of IPv6 addresses or networks. Must be in IP/mask format.
        """
        return pulumi.get(self, "ipv6")

    @_builtins.property
    @pulumi.getter
    def label(self) -> _builtins.str:
        """
        Used to identify this rule. For display purposes only.
        """
        return pulumi.get(self, "label")

    @_builtins.property
    @pulumi.getter
    def region(self) -> _builtins.str:
        """
        The Region where this Linode NodeBalancer is located. NodeBalancers only support backends in the same Region.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Sequence[_builtins.str]:
        """
        The tags applied to the firewall. Tags are case-insensitive and are for organizational purposes only.
        """
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter
    def transfers(self) -> Sequence['outputs.GetNodeBalancerTransferResult']:
        return pulumi.get(self, "transfers")

    @_builtins.property
    @pulumi.getter
    def updated(self) -> _builtins.str:
        """
        When this firewall was last updated.
        """
        return pulumi.get(self, "updated")

    @_builtins.property
    @pulumi.getter
    def vpcs(self) -> Sequence['outputs.GetNodeBalancerVpcResult']:
        return pulumi.get(self, "vpcs")


class AwaitableGetNodeBalancerResult(GetNodeBalancerResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetNodeBalancerResult(
            client_conn_throttle=self.client_conn_throttle,
            client_udp_sess_throttle=self.client_udp_sess_throttle,
            created=self.created,
            firewalls=self.firewalls,
            hostname=self.hostname,
            id=self.id,
            ipv4=self.ipv4,
            ipv6=self.ipv6,
            label=self.label,
            region=self.region,
            tags=self.tags,
            transfers=self.transfers,
            updated=self.updated,
            vpcs=self.vpcs)


def get_node_balancer(firewalls: Optional[Sequence[Union['GetNodeBalancerFirewallArgs', 'GetNodeBalancerFirewallArgsDict']]] = None,
                      id: Optional[_builtins.int] = None,
                      opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetNodeBalancerResult:
    """
    Provides details about a Linode NodeBalancer.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-node-balancer).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_linode as linode

    my_nodebalancer = linode.get_node_balancer(id=123)
    ```


    :param _builtins.int id: The NodeBalancer's ID.
    """
    __args__ = dict()
    __args__['firewalls'] = firewalls
    __args__['id'] = id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('linode:index/getNodeBalancer:getNodeBalancer', __args__, opts=opts, typ=GetNodeBalancerResult).value

    return AwaitableGetNodeBalancerResult(
        client_conn_throttle=pulumi.get(__ret__, 'client_conn_throttle'),
        client_udp_sess_throttle=pulumi.get(__ret__, 'client_udp_sess_throttle'),
        created=pulumi.get(__ret__, 'created'),
        firewalls=pulumi.get(__ret__, 'firewalls'),
        hostname=pulumi.get(__ret__, 'hostname'),
        id=pulumi.get(__ret__, 'id'),
        ipv4=pulumi.get(__ret__, 'ipv4'),
        ipv6=pulumi.get(__ret__, 'ipv6'),
        label=pulumi.get(__ret__, 'label'),
        region=pulumi.get(__ret__, 'region'),
        tags=pulumi.get(__ret__, 'tags'),
        transfers=pulumi.get(__ret__, 'transfers'),
        updated=pulumi.get(__ret__, 'updated'),
        vpcs=pulumi.get(__ret__, 'vpcs'))
def get_node_balancer_output(firewalls: Optional[pulumi.Input[Optional[Sequence[Union['GetNodeBalancerFirewallArgs', 'GetNodeBalancerFirewallArgsDict']]]]] = None,
                             id: Optional[pulumi.Input[_builtins.int]] = None,
                             opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetNodeBalancerResult]:
    """
    Provides details about a Linode NodeBalancer.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-node-balancer).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_linode as linode

    my_nodebalancer = linode.get_node_balancer(id=123)
    ```


    :param _builtins.int id: The NodeBalancer's ID.
    """
    __args__ = dict()
    __args__['firewalls'] = firewalls
    __args__['id'] = id
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('linode:index/getNodeBalancer:getNodeBalancer', __args__, opts=opts, typ=GetNodeBalancerResult)
    return __ret__.apply(lambda __response__: GetNodeBalancerResult(
        client_conn_throttle=pulumi.get(__response__, 'client_conn_throttle'),
        client_udp_sess_throttle=pulumi.get(__response__, 'client_udp_sess_throttle'),
        created=pulumi.get(__response__, 'created'),
        firewalls=pulumi.get(__response__, 'firewalls'),
        hostname=pulumi.get(__response__, 'hostname'),
        id=pulumi.get(__response__, 'id'),
        ipv4=pulumi.get(__response__, 'ipv4'),
        ipv6=pulumi.get(__response__, 'ipv6'),
        label=pulumi.get(__response__, 'label'),
        region=pulumi.get(__response__, 'region'),
        tags=pulumi.get(__response__, 'tags'),
        transfers=pulumi.get(__response__, 'transfers'),
        updated=pulumi.get(__response__, 'updated'),
        vpcs=pulumi.get(__response__, 'vpcs')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetNodeBalancerConfigResult',
    'AwaitableGetNodeBalancerConfigResult',
    'get_node_balancer_config',
    'get_node_balancer_config_output',
]

@pulumi.output_type
class GetNodeBalancerConfigResult:
    """
    A collection of values returned by getNodeBalancerConfig.
    """
    def __init__(__self__, algorithm=None, check=None, check_attempts=None, check_body=None, check_interval=None, check_passive=None, check_path=None, check_timeout=None, cipher_suite=None, id=None, node_statuses=None, nodebalancer_id=None, port=None, protocol=None, proxy_protocol=None, ssl_commonname=None, ssl_fingerprint=None, stickiness=None, udp_check_port=None, udp_session_timeout=None):
        if algorithm and not isinstance(algorithm, str):
            raise TypeError("Expected argument 'algorithm' to be a str")
        pulumi.set(__self__, "algorithm", algorithm)
        if check and not isinstance(check, str):
            raise TypeError("Expected argument 'check' to be a str")
        pulumi.set(__self__, "check", check)
        if check_attempts and not isinstance(check_attempts, int):
            raise TypeError("Expected argument 'check_attempts' to be a int")
        pulumi.set(__self__, "check_attempts", check_attempts)
        if check_body and not isinstance(check_body, str):
            raise TypeError("Expected argument 'check_body' to be a str")
        pulumi.set(__self__, "check_body", check_body)
        if check_interval and not isinstance(check_interval, int):
            raise TypeError("Expected argument 'check_interval' to be a int")
        pulumi.set(__self__, "check_interval", check_interval)
        if check_passive and not isinstance(check_passive, bool):
            raise TypeError("Expected argument 'check_passive' to be a bool")
        pulumi.set(__self__, "check_passive", check_passive)
        if check_path and not isinstance(check_path, str):
            raise TypeError("Expected argument 'check_path' to be a str")
        pulumi.set(__self__, "check_path", check_path)
        if check_timeout and not isinstance(check_timeout, int):
            raise TypeError("Expected argument 'check_timeout' to be a int")
        pulumi.set(__self__, "check_timeout", check_timeout)
        if cipher_suite and not isinstance(cipher_suite, str):
            raise TypeError("Expected argument 'cipher_suite' to be a str")
        pulumi.set(__self__, "cipher_suite", cipher_suite)
        if id and not isinstance(id, int):
            raise TypeError("Expected argument 'id' to be a int")
        pulumi.set(__self__, "id", id)
        if node_statuses and not isinstance(node_statuses, list):
            raise TypeError("Expected argument 'node_statuses' to be a list")
        pulumi.set(__self__, "node_statuses", node_statuses)
        if nodebalancer_id and not isinstance(nodebalancer_id, int):
            raise TypeError("Expected argument 'nodebalancer_id' to be a int")
        pulumi.set(__self__, "nodebalancer_id", nodebalancer_id)
        if port and not isinstance(port, int):
            raise TypeError("Expected argument 'port' to be a int")
        pulumi.set(__self__, "port", port)
        if protocol and not isinstance(protocol, str):
            raise TypeError("Expected argument 'protocol' to be a str")
        pulumi.set(__self__, "protocol", protocol)
        if proxy_protocol and not isinstance(proxy_protocol, str):
            raise TypeError("Expected argument 'proxy_protocol' to be a str")
        pulumi.set(__self__, "proxy_protocol", proxy_protocol)
        if ssl_commonname and not isinstance(ssl_commonname, str):
            raise TypeError("Expected argument 'ssl_commonname' to be a str")
        pulumi.set(__self__, "ssl_commonname", ssl_commonname)
        if ssl_fingerprint and not isinstance(ssl_fingerprint, str):
            raise TypeError("Expected argument 'ssl_fingerprint' to be a str")
        pulumi.set(__self__, "ssl_fingerprint", ssl_fingerprint)
        if stickiness and not isinstance(stickiness, str):
            raise TypeError("Expected argument 'stickiness' to be a str")
        pulumi.set(__self__, "stickiness", stickiness)
        if udp_check_port and not isinstance(udp_check_port, int):
            raise TypeError("Expected argument 'udp_check_port' to be a int")
        pulumi.set(__self__, "udp_check_port", udp_check_port)
        if udp_session_timeout and not isinstance(udp_session_timeout, int):
            raise TypeError("Expected argument 'udp_session_timeout' to be a int")
        pulumi.set(__self__, "udp_session_timeout", udp_session_timeout)

    @_builtins.property
    @pulumi.getter
    def algorithm(self) -> _builtins.str:
        """
        What algorithm this NodeBalancer should use for routing traffic to backends (`roundrobin`, `leastconn`, `source`)
        """
        return pulumi.get(self, "algorithm")

    @_builtins.property
    @pulumi.getter
    def check(self) -> _builtins.str:
        """
        The type of check to perform against backends to ensure they are serving requests. This is used to determine if backends are up or down. If none no check is performed. connection requires only a connection to the backend to succeed. http and http_body rely on the backend serving HTTP, and that the response returned matches what is expected. (`none`, `connection`, `http`, `http_body`)
        """
        return pulumi.get(self, "check")

    @_builtins.property
    @pulumi.getter(name="checkAttempts")
    def check_attempts(self) -> _builtins.int:
        """
        How many times to attempt a check before considering a backend to be down. (1-30)
        """
        return pulumi.get(self, "check_attempts")

    @_builtins.property
    @pulumi.getter(name="checkBody")
    def check_body(self) -> _builtins.str:
        return pulumi.get(self, "check_body")

    @_builtins.property
    @pulumi.getter(name="checkInterval")
    def check_interval(self) -> _builtins.int:
        """
        How often, in seconds, to check that backends are up and serving requests.
        """
        return pulumi.get(self, "check_interval")

    @_builtins.property
    @pulumi.getter(name="checkPassive")
    def check_passive(self) -> _builtins.bool:
        """
        If true, any response from this backend with a 5xx status code will be enough for it to be considered unhealthy and taken out of rotation.
        """
        return pulumi.get(self, "check_passive")

    @_builtins.property
    @pulumi.getter(name="checkPath")
    def check_path(self) -> _builtins.str:
        """
        The URL path to check on each backend. If the backend does not respond to this request it is considered to be down.
        """
        return pulumi.get(self, "check_path")

    @_builtins.property
    @pulumi.getter(name="checkTimeout")
    def check_timeout(self) -> _builtins.int:
        """
        How long, in seconds, to wait for a check attempt before considering it failed. (1-30)
        """
        return pulumi.get(self, "check_timeout")

    @_builtins.property
    @pulumi.getter(name="cipherSuite")
    def cipher_suite(self) -> _builtins.str:
        """
        What ciphers to use for SSL connections served by this NodeBalancer. `legacy` is considered insecure and should only be used if necessary. (`recommended`, `legacy`)
        """
        return pulumi.get(self, "cipher_suite")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.int:
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="nodeStatuses")
    def node_statuses(self) -> Sequence['outputs.GetNodeBalancerConfigNodeStatusResult']:
        return pulumi.get(self, "node_statuses")

    @_builtins.property
    @pulumi.getter(name="nodebalancerId")
    def nodebalancer_id(self) -> _builtins.int:
        return pulumi.get(self, "nodebalancer_id")

    @_builtins.property
    @pulumi.getter
    def port(self) -> _builtins.int:
        """
        The TCP port this Config is for.
        """
        return pulumi.get(self, "port")

    @_builtins.property
    @pulumi.getter
    def protocol(self) -> _builtins.str:
        """
        The protocol this port is configured to serve. If this is set to https you must include an ssl_cert and an ssl_key. (`http`, `https`, `tcp`) (Defaults to `http`)
        """
        return pulumi.get(self, "protocol")

    @_builtins.property
    @pulumi.getter(name="proxyProtocol")
    def proxy_protocol(self) -> _builtins.str:
        """
        The version of ProxyProtocol to use for the underlying NodeBalancer. This requires protocol to be `tcp`. (`none`, `v1`, and `v2`) (Defaults to `none`)
        """
        return pulumi.get(self, "proxy_protocol")

    @_builtins.property
    @pulumi.getter(name="sslCommonname")
    def ssl_commonname(self) -> _builtins.str:
        """
        The read-only common name automatically derived from the SSL certificate assigned to this NodeBalancerConfig. Please refer to this field to verify that the appropriate certificate is assigned to your NodeBalancerConfig.
        """
        return pulumi.get(self, "ssl_commonname")

    @_builtins.property
    @pulumi.getter(name="sslFingerprint")
    def ssl_fingerprint(self) -> _builtins.str:
        """
        The read-only fingerprint automatically derived from the SSL certificate assigned to this NodeBalancerConfig. Please refer to this field to verify that the appropriate certificate is assigned to your NodeBalancerConfig.
        """
        return pulumi.get(self, "ssl_fingerprint")

    @_builtins.property
    @pulumi.getter
    def stickiness(self) -> _builtins.str:
        """
        Controls how session stickiness is handled on this port. (`none`, `table`, `http_cookie`)
        """
        return pulumi.get(self, "stickiness")

    @_builtins.property
    @pulumi.getter(name="udpCheckPort")
    def udp_check_port(self) -> _builtins.int:
        """
        Specifies the port on the backend node used for active health checks, which may differ from the port serving traffic.
        """
        return pulumi.get(self, "udp_check_port")

    @_builtins.property
    @pulumi.getter(name="udpSessionTimeout")
    def udp_session_timeout(self) -> _builtins.int:
        """
        The idle time in seconds after which a session that hasn’t received packets is destroyed.
        """
        return pulumi.get(self, "udp_session_timeout")


class AwaitableGetNodeBalancerConfigResult(GetNodeBalancerConfigResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetNodeBalancerConfigResult(
            algorithm=self.algorithm,
            check=self.check,
            check_attempts=self.check_attempts,
            check_body=self.check_body,
            check_interval=self.check_interval,
            check_passive=self.check_passive,
            check_path=self.check_path,
            check_timeout=self.check_timeout,
            cipher_suite=self.cipher_suite,
            id=self.id,
            node_statuses=self.node_statuses,
            nodebalancer_id=self.nodebalancer_id,
            port=self.port,
            protocol=self.protocol,
            proxy_protocol=self.proxy_protocol,
            ssl_commonname=self.ssl_commonname,
            ssl_fingerprint=self.ssl_fingerprint,
            stickiness=self.stickiness,
            udp_check_port=self.udp_check_port,
            udp_session_timeout=self.udp_session_timeout)


def get_node_balancer_config(id: Optional[_builtins.int] = None,
                             nodebalancer_id: Optional[_builtins.int] = None,
                             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetNodeBalancerConfigResult:
    """
    Provides details about a Linode NodeBalancer Config.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-node-balancer-config).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_linode as linode

    my_config = linode.get_node_balancer_config(id=123,
        nodebalancer_id=456)
    ```


    :param _builtins.int id: The config's ID.
    :param _builtins.int nodebalancer_id: The ID of the NodeBalancer that contains the config.
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['nodebalancerId'] = nodebalancer_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('linode:index/getNodeBalancerConfig:getNodeBalancerConfig', __args__, opts=opts, typ=GetNodeBalancerConfigResult).value

    return AwaitableGetNodeBalancerConfigResult(
        algorithm=pulumi.get(__ret__, 'algorithm'),
        check=pulumi.get(__ret__, 'check'),
        check_attempts=pulumi.get(__ret__, 'check_attempts'),
        check_body=pulumi.get(__ret__, 'check_body'),
        check_interval=pulumi.get(__ret__, 'check_interval'),
        check_passive=pulumi.get(__ret__, 'check_passive'),
        check_path=pulumi.get(__ret__, 'check_path'),
        check_timeout=pulumi.get(__ret__, 'check_timeout'),
        cipher_suite=pulumi.get(__ret__, 'cipher_suite'),
        id=pulumi.get(__ret__, 'id'),
        node_statuses=pulumi.get(__ret__, 'node_statuses'),
        nodebalancer_id=pulumi.get(__ret__, 'nodebalancer_id'),
        port=pulumi.get(__ret__, 'port'),
        protocol=pulumi.get(__ret__, 'protocol'),
        proxy_protocol=pulumi.get(__ret__, 'proxy_protocol'),
        ssl_commonname=pulumi.get(__ret__, 'ssl_commonname'),
        ssl_fingerprint=pulumi.get(__ret__, 'ssl_fingerprint'),
        stickiness=pulumi.get(__ret__, 'stickiness'),
        udp_check_port=pulumi.get(__ret__, 'udp_check_port'),
        udp_session_timeout=pulumi.get(__ret__, 'udp_session_timeout'))
def get_node_balancer_config_output(id: Optional[pulumi.Input[_builtins.int]] = None,
                                    nodebalancer_id: Optional[pulumi.Input[_builtins.int]] = None,
                                    opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetNodeBalancerConfigResult]:
    """
    Provides details about a Linode NodeBalancer Config.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-node-balancer-config).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_linode as linode

    my_config = linode.get_node_balancer_config(id=123,
        nodebalancer_id=456)
    ```


    :param _builtins.int id: The config's ID.
    :param _builtins.int nodebalancer_id: The ID of the NodeBalancer that contains the config.
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['nodebalancerId'] = nodebalancer_id
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('linode:index/getNodeBalancerConfig:getNodeBalancerConfig', __args__, opts=opts, typ=GetNodeBalancerConfigResult)
    return __ret__.apply(lambda __response__: GetNodeBalancerConfigResult(
        algorithm=pulumi.get(__response__, 'algorithm'),
        check=pulumi.get(__response__, 'check'),
        check_attempts=pulumi.get(__response__, 'check_attempts'),
        check_body=pulumi.get(__response__, 'check_body'),
        check_interval=pulumi.get(__response__, 'check_interval'),
        check_passive=pulumi.get(__response__, 'check_passive'),
        check_path=pulumi.get(__response__, 'check_path'),
        check_timeout=pulumi.get(__response__, 'check_timeout'),
        cipher_suite=pulumi.get(__response__, 'cipher_suite'),
        id=pulumi.get(__response__, 'id'),
        node_statuses=pulumi.get(__response__, 'node_statuses'),
        nodebalancer_id=pulumi.get(__response__, 'nodebalancer_id'),
        port=pulumi.get(__response__, 'port'),
        protocol=pulumi.get(__response__, 'protocol'),
        proxy_protocol=pulumi.get(__response__, 'proxy_protocol'),
        ssl_commonname=pulumi.get(__response__, 'ssl_commonname'),
        ssl_fingerprint=pulumi.get(__response__, 'ssl_fingerprint'),
        stickiness=pulumi.get(__response__, 'stickiness'),
        udp_check_port=pulumi.get(__response__, 'udp_check_port'),
        udp_session_timeout=pulumi.get(__response__, 'udp_session_timeout')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetFirewallTemplatesResult',
    'AwaitableGetFirewallTemplatesResult',
    'get_firewall_templates',
    'get_firewall_templates_output',
]

@pulumi.output_type
class GetFirewallTemplatesResult:
    """
    A collection of values returned by getFirewallTemplates.
    """
    def __init__(__self__, filters=None, firewall_templates=None, id=None):
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if firewall_templates and not isinstance(firewall_templates, list):
            raise TypeError("Expected argument 'firewall_templates' to be a list")
        pulumi.set(__self__, "firewall_templates", firewall_templates)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)

    @_builtins.property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetFirewallTemplatesFilterResult']]:
        return pulumi.get(self, "filters")

    @_builtins.property
    @pulumi.getter(name="firewallTemplates")
    def firewall_templates(self) -> Optional[Sequence['outputs.GetFirewallTemplatesFirewallTemplateResult']]:
        return pulumi.get(self, "firewall_templates")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        return pulumi.get(self, "id")


class AwaitableGetFirewallTemplatesResult(GetFirewallTemplatesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetFirewallTemplatesResult(
            filters=self.filters,
            firewall_templates=self.firewall_templates,
            id=self.id)


def get_firewall_templates(filters: Optional[Sequence[Union['GetFirewallTemplatesFilterArgs', 'GetFirewallTemplatesFilterArgsDict']]] = None,
                           firewall_templates: Optional[Sequence[Union['GetFirewallTemplatesFirewallTemplateArgs', 'GetFirewallTemplatesFirewallTemplateArgsDict']]] = None,
                           opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetFirewallTemplatesResult:
    """
    Provides information about all Linode Firewall Templates.

    ## Example Usage

    The following example shows how one might use this data source to list all available Firewall Templates:

    ```python
    import pulumi
    import pulumi_linode as linode

    all = linode.get_firewall_templates()
    pulumi.export("firewallTemplateSlugs", all.firewall_templates)
    ```

    Or with some filters to get a subset of the results.

    ```python
    import pulumi
    import pulumi_linode as linode

    filtered = linode.get_firewall_templates(filters=[{
        "name": "slug",
        "values": ["public"],
        "match_by": "exact",
    }])
    pulumi.export("firewallTemplateSlugs", filtered.firewall_templates)
    ```

    ## Filterable Fields

    * `slug`
    """
    __args__ = dict()
    __args__['filters'] = filters
    __args__['firewallTemplates'] = firewall_templates
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('linode:index/getFirewallTemplates:getFirewallTemplates', __args__, opts=opts, typ=GetFirewallTemplatesResult).value

    return AwaitableGetFirewallTemplatesResult(
        filters=pulumi.get(__ret__, 'filters'),
        firewall_templates=pulumi.get(__ret__, 'firewall_templates'),
        id=pulumi.get(__ret__, 'id'))
def get_firewall_templates_output(filters: Optional[pulumi.Input[Optional[Sequence[Union['GetFirewallTemplatesFilterArgs', 'GetFirewallTemplatesFilterArgsDict']]]]] = None,
                                  firewall_templates: Optional[pulumi.Input[Optional[Sequence[Union['GetFirewallTemplatesFirewallTemplateArgs', 'GetFirewallTemplatesFirewallTemplateArgsDict']]]]] = None,
                                  opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetFirewallTemplatesResult]:
    """
    Provides information about all Linode Firewall Templates.

    ## Example Usage

    The following example shows how one might use this data source to list all available Firewall Templates:

    ```python
    import pulumi
    import pulumi_linode as linode

    all = linode.get_firewall_templates()
    pulumi.export("firewallTemplateSlugs", all.firewall_templates)
    ```

    Or with some filters to get a subset of the results.

    ```python
    import pulumi
    import pulumi_linode as linode

    filtered = linode.get_firewall_templates(filters=[{
        "name": "slug",
        "values": ["public"],
        "match_by": "exact",
    }])
    pulumi.export("firewallTemplateSlugs", filtered.firewall_templates)
    ```

    ## Filterable Fields

    * `slug`
    """
    __args__ = dict()
    __args__['filters'] = filters
    __args__['firewallTemplates'] = firewall_templates
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('linode:index/getFirewallTemplates:getFirewallTemplates', __args__, opts=opts, typ=GetFirewallTemplatesResult)
    return __ret__.apply(lambda __response__: GetFirewallTemplatesResult(
        filters=pulumi.get(__response__, 'filters'),
        firewall_templates=pulumi.get(__response__, 'firewall_templates'),
        id=pulumi.get(__response__, 'id')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['LkeClusterArgs', 'LkeCluster']

@pulumi.input_type
class LkeClusterArgs:
    def __init__(__self__, *,
                 k8s_version: pulumi.Input[_builtins.str],
                 label: pulumi.Input[_builtins.str],
                 pools: pulumi.Input[Sequence[pulumi.Input['LkeClusterPoolArgs']]],
                 region: pulumi.Input[_builtins.str],
                 apl_enabled: Optional[pulumi.Input[_builtins.bool]] = None,
                 control_plane: Optional[pulumi.Input['LkeClusterControlPlaneArgs']] = None,
                 external_pool_tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 stack_type: Optional[pulumi.Input[_builtins.str]] = None,
                 subnet_id: Optional[pulumi.Input[_builtins.int]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 tier: Optional[pulumi.Input[_builtins.str]] = None,
                 vpc_id: Optional[pulumi.Input[_builtins.int]] = None):
        """
        The set of arguments for constructing a LkeCluster resource.
        :param pulumi.Input[_builtins.str] k8s_version: The desired Kubernetes version for this Kubernetes cluster in the format of `major.minor` (e.g. `1.21`), and the latest supported patch version will be deployed.
        :param pulumi.Input[_builtins.str] label: This Kubernetes cluster's unique label.
        :param pulumi.Input[Sequence[pulumi.Input['LkeClusterPoolArgs']]] pools: Additional nested attributes:
        :param pulumi.Input[_builtins.str] region: This Kubernetes cluster's location.
               
               * `pool` - (Required) The Node Pool specifications for the Kubernetes cluster. At least one Node Pool is required.
               
               * `control_plane` (Optional) Defines settings for the Kubernetes Control Plane.
        :param pulumi.Input[_builtins.bool] apl_enabled: Enables the App Platform Layer
        :param pulumi.Input['LkeClusterControlPlaneArgs'] control_plane: Defines settings for the Kubernetes Control Plane.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] external_pool_tags: A set of node pool tags to ignore when planning and applying this cluster. This prevents externally managed node pools from being deleted or unintentionally updated on subsequent applies. See Externally Managed Node Pools for more details.
        :param pulumi.Input[_builtins.str] stack_type: The networking stack type of the Kubernetes cluster.
        :param pulumi.Input[_builtins.int] subnet_id: The ID of the VPC subnet to use for the Kubernetes cluster. This subnet must be dual stack (IPv4 and IPv6 should both be enabled). (**Note: v4beta only and may not currently be available to all users.**)
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: An array of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        :param pulumi.Input[_builtins.str] tier: The desired Kubernetes tier. (**Note: v4beta only and may not currently be available to all users.**)
        :param pulumi.Input[_builtins.int] vpc_id: The ID of the VPC to use for the Kubernetes cluster.
        """
        pulumi.set(__self__, "k8s_version", k8s_version)
        pulumi.set(__self__, "label", label)
        pulumi.set(__self__, "pools", pools)
        pulumi.set(__self__, "region", region)
        if apl_enabled is not None:
            pulumi.set(__self__, "apl_enabled", apl_enabled)
        if control_plane is not None:
            pulumi.set(__self__, "control_plane", control_plane)
        if external_pool_tags is not None:
            pulumi.set(__self__, "external_pool_tags", external_pool_tags)
        if stack_type is not None:
            pulumi.set(__self__, "stack_type", stack_type)
        if subnet_id is not None:
            pulumi.set(__self__, "subnet_id", subnet_id)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if tier is not None:
            pulumi.set(__self__, "tier", tier)
        if vpc_id is not None:
            pulumi.set(__self__, "vpc_id", vpc_id)

    @_builtins.property
    @pulumi.getter(name="k8sVersion")
    def k8s_version(self) -> pulumi.Input[_builtins.str]:
        """
        The desired Kubernetes version for this Kubernetes cluster in the format of `major.minor` (e.g. `1.21`), and the latest supported patch version will be deployed.
        """
        return pulumi.get(self, "k8s_version")

    @k8s_version.setter
    def k8s_version(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "k8s_version", value)

    @_builtins.property
    @pulumi.getter
    def label(self) -> pulumi.Input[_builtins.str]:
        """
        This Kubernetes cluster's unique label.
        """
        return pulumi.get(self, "label")

    @label.setter
    def label(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "label", value)

    @_builtins.property
    @pulumi.getter
    def pools(self) -> pulumi.Input[Sequence[pulumi.Input['LkeClusterPoolArgs']]]:
        """
        Additional nested attributes:
        """
        return pulumi.get(self, "pools")

    @pools.setter
    def pools(self, value: pulumi.Input[Sequence[pulumi.Input['LkeClusterPoolArgs']]]):
        pulumi.set(self, "pools", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Input[_builtins.str]:
        """
        This Kubernetes cluster's location.

        * `pool` - (Required) The Node Pool specifications for the Kubernetes cluster. At least one Node Pool is required.

        * `control_plane` (Optional) Defines settings for the Kubernetes Control Plane.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter(name="aplEnabled")
    def apl_enabled(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Enables the App Platform Layer
        """
        return pulumi.get(self, "apl_enabled")

    @apl_enabled.setter
    def apl_enabled(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "apl_enabled", value)

    @_builtins.property
    @pulumi.getter(name="controlPlane")
    def control_plane(self) -> Optional[pulumi.Input['LkeClusterControlPlaneArgs']]:
        """
        Defines settings for the Kubernetes Control Plane.
        """
        return pulumi.get(self, "control_plane")

    @control_plane.setter
    def control_plane(self, value: Optional[pulumi.Input['LkeClusterControlPlaneArgs']]):
        pulumi.set(self, "control_plane", value)

    @_builtins.property
    @pulumi.getter(name="externalPoolTags")
    def external_pool_tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A set of node pool tags to ignore when planning and applying this cluster. This prevents externally managed node pools from being deleted or unintentionally updated on subsequent applies. See Externally Managed Node Pools for more details.
        """
        return pulumi.get(self, "external_pool_tags")

    @external_pool_tags.setter
    def external_pool_tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "external_pool_tags", value)

    @_builtins.property
    @pulumi.getter(name="stackType")
    def stack_type(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The networking stack type of the Kubernetes cluster.
        """
        return pulumi.get(self, "stack_type")

    @stack_type.setter
    def stack_type(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "stack_type", value)

    @_builtins.property
    @pulumi.getter(name="subnetId")
    def subnet_id(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The ID of the VPC subnet to use for the Kubernetes cluster. This subnet must be dual stack (IPv4 and IPv6 should both be enabled). (**Note: v4beta only and may not currently be available to all users.**)
        """
        return pulumi.get(self, "subnet_id")

    @subnet_id.setter
    def subnet_id(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "subnet_id", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        An array of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter
    def tier(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The desired Kubernetes tier. (**Note: v4beta only and may not currently be available to all users.**)
        """
        return pulumi.get(self, "tier")

    @tier.setter
    def tier(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "tier", value)

    @_builtins.property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The ID of the VPC to use for the Kubernetes cluster.
        """
        return pulumi.get(self, "vpc_id")

    @vpc_id.setter
    def vpc_id(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "vpc_id", value)


@pulumi.input_type
class _LkeClusterState:
    def __init__(__self__, *,
                 api_endpoints: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 apl_enabled: Optional[pulumi.Input[_builtins.bool]] = None,
                 control_plane: Optional[pulumi.Input['LkeClusterControlPlaneArgs']] = None,
                 dashboard_url: Optional[pulumi.Input[_builtins.str]] = None,
                 external_pool_tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 k8s_version: Optional[pulumi.Input[_builtins.str]] = None,
                 kubeconfig: Optional[pulumi.Input[_builtins.str]] = None,
                 label: Optional[pulumi.Input[_builtins.str]] = None,
                 pools: Optional[pulumi.Input[Sequence[pulumi.Input['LkeClusterPoolArgs']]]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 stack_type: Optional[pulumi.Input[_builtins.str]] = None,
                 status: Optional[pulumi.Input[_builtins.str]] = None,
                 subnet_id: Optional[pulumi.Input[_builtins.int]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 tier: Optional[pulumi.Input[_builtins.str]] = None,
                 vpc_id: Optional[pulumi.Input[_builtins.int]] = None):
        """
        Input properties used for looking up and filtering LkeCluster resources.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] api_endpoints: The endpoints for the Kubernetes API server.
        :param pulumi.Input[_builtins.bool] apl_enabled: Enables the App Platform Layer
        :param pulumi.Input['LkeClusterControlPlaneArgs'] control_plane: Defines settings for the Kubernetes Control Plane.
        :param pulumi.Input[_builtins.str] dashboard_url: The Kubernetes Dashboard access URL for this cluster. LKE Enterprise does not have a dashboard URL.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] external_pool_tags: A set of node pool tags to ignore when planning and applying this cluster. This prevents externally managed node pools from being deleted or unintentionally updated on subsequent applies. See Externally Managed Node Pools for more details.
        :param pulumi.Input[_builtins.str] k8s_version: The desired Kubernetes version for this Kubernetes cluster in the format of `major.minor` (e.g. `1.21`), and the latest supported patch version will be deployed.
        :param pulumi.Input[_builtins.str] kubeconfig: The base64 encoded kubeconfig for the Kubernetes cluster.
        :param pulumi.Input[_builtins.str] label: This Kubernetes cluster's unique label.
        :param pulumi.Input[Sequence[pulumi.Input['LkeClusterPoolArgs']]] pools: Additional nested attributes:
        :param pulumi.Input[_builtins.str] region: This Kubernetes cluster's location.
               
               * `pool` - (Required) The Node Pool specifications for the Kubernetes cluster. At least one Node Pool is required.
               
               * `control_plane` (Optional) Defines settings for the Kubernetes Control Plane.
        :param pulumi.Input[_builtins.str] stack_type: The networking stack type of the Kubernetes cluster.
        :param pulumi.Input[_builtins.str] status: The status of the node. (`ready`, `not_ready`)
        :param pulumi.Input[_builtins.int] subnet_id: The ID of the VPC subnet to use for the Kubernetes cluster. This subnet must be dual stack (IPv4 and IPv6 should both be enabled). (**Note: v4beta only and may not currently be available to all users.**)
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: An array of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        :param pulumi.Input[_builtins.str] tier: The desired Kubernetes tier. (**Note: v4beta only and may not currently be available to all users.**)
        :param pulumi.Input[_builtins.int] vpc_id: The ID of the VPC to use for the Kubernetes cluster.
        """
        if api_endpoints is not None:
            pulumi.set(__self__, "api_endpoints", api_endpoints)
        if apl_enabled is not None:
            pulumi.set(__self__, "apl_enabled", apl_enabled)
        if control_plane is not None:
            pulumi.set(__self__, "control_plane", control_plane)
        if dashboard_url is not None:
            pulumi.set(__self__, "dashboard_url", dashboard_url)
        if external_pool_tags is not None:
            pulumi.set(__self__, "external_pool_tags", external_pool_tags)
        if k8s_version is not None:
            pulumi.set(__self__, "k8s_version", k8s_version)
        if kubeconfig is not None:
            pulumi.set(__self__, "kubeconfig", kubeconfig)
        if label is not None:
            pulumi.set(__self__, "label", label)
        if pools is not None:
            pulumi.set(__self__, "pools", pools)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if stack_type is not None:
            pulumi.set(__self__, "stack_type", stack_type)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if subnet_id is not None:
            pulumi.set(__self__, "subnet_id", subnet_id)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if tier is not None:
            pulumi.set(__self__, "tier", tier)
        if vpc_id is not None:
            pulumi.set(__self__, "vpc_id", vpc_id)

    @_builtins.property
    @pulumi.getter(name="apiEndpoints")
    def api_endpoints(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The endpoints for the Kubernetes API server.
        """
        return pulumi.get(self, "api_endpoints")

    @api_endpoints.setter
    def api_endpoints(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "api_endpoints", value)

    @_builtins.property
    @pulumi.getter(name="aplEnabled")
    def apl_enabled(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Enables the App Platform Layer
        """
        return pulumi.get(self, "apl_enabled")

    @apl_enabled.setter
    def apl_enabled(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "apl_enabled", value)

    @_builtins.property
    @pulumi.getter(name="controlPlane")
    def control_plane(self) -> Optional[pulumi.Input['LkeClusterControlPlaneArgs']]:
        """
        Defines settings for the Kubernetes Control Plane.
        """
        return pulumi.get(self, "control_plane")

    @control_plane.setter
    def control_plane(self, value: Optional[pulumi.Input['LkeClusterControlPlaneArgs']]):
        pulumi.set(self, "control_plane", value)

    @_builtins.property
    @pulumi.getter(name="dashboardUrl")
    def dashboard_url(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The Kubernetes Dashboard access URL for this cluster. LKE Enterprise does not have a dashboard URL.
        """
        return pulumi.get(self, "dashboard_url")

    @dashboard_url.setter
    def dashboard_url(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "dashboard_url", value)

    @_builtins.property
    @pulumi.getter(name="externalPoolTags")
    def external_pool_tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A set of node pool tags to ignore when planning and applying this cluster. This prevents externally managed node pools from being deleted or unintentionally updated on subsequent applies. See Externally Managed Node Pools for more details.
        """
        return pulumi.get(self, "external_pool_tags")

    @external_pool_tags.setter
    def external_pool_tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "external_pool_tags", value)

    @_builtins.property
    @pulumi.getter(name="k8sVersion")
    def k8s_version(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The desired Kubernetes version for this Kubernetes cluster in the format of `major.minor` (e.g. `1.21`), and the latest supported patch version will be deployed.
        """
        return pulumi.get(self, "k8s_version")

    @k8s_version.setter
    def k8s_version(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "k8s_version", value)

    @_builtins.property
    @pulumi.getter
    def kubeconfig(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The base64 encoded kubeconfig for the Kubernetes cluster.
        """
        return pulumi.get(self, "kubeconfig")

    @kubeconfig.setter
    def kubeconfig(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "kubeconfig", value)

    @_builtins.property
    @pulumi.getter
    def label(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        This Kubernetes cluster's unique label.
        """
        return pulumi.get(self, "label")

    @label.setter
    def label(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "label", value)

    @_builtins.property
    @pulumi.getter
    def pools(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['LkeClusterPoolArgs']]]]:
        """
        Additional nested attributes:
        """
        return pulumi.get(self, "pools")

    @pools.setter
    def pools(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['LkeClusterPoolArgs']]]]):
        pulumi.set(self, "pools", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        This Kubernetes cluster's location.

        * `pool` - (Required) The Node Pool specifications for the Kubernetes cluster. At least one Node Pool is required.

        * `control_plane` (Optional) Defines settings for the Kubernetes Control Plane.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter(name="stackType")
    def stack_type(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The networking stack type of the Kubernetes cluster.
        """
        return pulumi.get(self, "stack_type")

    @stack_type.setter
    def stack_type(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "stack_type", value)

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The status of the node. (`ready`, `not_ready`)
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "status", value)

    @_builtins.property
    @pulumi.getter(name="subnetId")
    def subnet_id(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The ID of the VPC subnet to use for the Kubernetes cluster. This subnet must be dual stack (IPv4 and IPv6 should both be enabled). (**Note: v4beta only and may not currently be available to all users.**)
        """
        return pulumi.get(self, "subnet_id")

    @subnet_id.setter
    def subnet_id(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "subnet_id", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        An array of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter
    def tier(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The desired Kubernetes tier. (**Note: v4beta only and may not currently be available to all users.**)
        """
        return pulumi.get(self, "tier")

    @tier.setter
    def tier(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "tier", value)

    @_builtins.property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The ID of the VPC to use for the Kubernetes cluster.
        """
        return pulumi.get(self, "vpc_id")

    @vpc_id.setter
    def vpc_id(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "vpc_id", value)


@pulumi.type_token("linode:index/lkeCluster:LkeCluster")
class LkeCluster(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 apl_enabled: Optional[pulumi.Input[_builtins.bool]] = None,
                 control_plane: Optional[pulumi.Input[Union['LkeClusterControlPlaneArgs', 'LkeClusterControlPlaneArgsDict']]] = None,
                 external_pool_tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 k8s_version: Optional[pulumi.Input[_builtins.str]] = None,
                 label: Optional[pulumi.Input[_builtins.str]] = None,
                 pools: Optional[pulumi.Input[Sequence[pulumi.Input[Union['LkeClusterPoolArgs', 'LkeClusterPoolArgsDict']]]]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 stack_type: Optional[pulumi.Input[_builtins.str]] = None,
                 subnet_id: Optional[pulumi.Input[_builtins.int]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 tier: Optional[pulumi.Input[_builtins.str]] = None,
                 vpc_id: Optional[pulumi.Input[_builtins.int]] = None,
                 __props__=None):
        """
        ## Import

        LKE Clusters can be imported using the `id`, e.g.

        ```sh
        $ pulumi import linode:index/lkeCluster:LkeCluster my_cluster 12345
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.bool] apl_enabled: Enables the App Platform Layer
        :param pulumi.Input[Union['LkeClusterControlPlaneArgs', 'LkeClusterControlPlaneArgsDict']] control_plane: Defines settings for the Kubernetes Control Plane.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] external_pool_tags: A set of node pool tags to ignore when planning and applying this cluster. This prevents externally managed node pools from being deleted or unintentionally updated on subsequent applies. See Externally Managed Node Pools for more details.
        :param pulumi.Input[_builtins.str] k8s_version: The desired Kubernetes version for this Kubernetes cluster in the format of `major.minor` (e.g. `1.21`), and the latest supported patch version will be deployed.
        :param pulumi.Input[_builtins.str] label: This Kubernetes cluster's unique label.
        :param pulumi.Input[Sequence[pulumi.Input[Union['LkeClusterPoolArgs', 'LkeClusterPoolArgsDict']]]] pools: Additional nested attributes:
        :param pulumi.Input[_builtins.str] region: This Kubernetes cluster's location.
               
               * `pool` - (Required) The Node Pool specifications for the Kubernetes cluster. At least one Node Pool is required.
               
               * `control_plane` (Optional) Defines settings for the Kubernetes Control Plane.
        :param pulumi.Input[_builtins.str] stack_type: The networking stack type of the Kubernetes cluster.
        :param pulumi.Input[_builtins.int] subnet_id: The ID of the VPC subnet to use for the Kubernetes cluster. This subnet must be dual stack (IPv4 and IPv6 should both be enabled). (**Note: v4beta only and may not currently be available to all users.**)
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: An array of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        :param pulumi.Input[_builtins.str] tier: The desired Kubernetes tier. (**Note: v4beta only and may not currently be available to all users.**)
        :param pulumi.Input[_builtins.int] vpc_id: The ID of the VPC to use for the Kubernetes cluster.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: LkeClusterArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Import

        LKE Clusters can be imported using the `id`, e.g.

        ```sh
        $ pulumi import linode:index/lkeCluster:LkeCluster my_cluster 12345
        ```

        :param str resource_name: The name of the resource.
        :param LkeClusterArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(LkeClusterArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 apl_enabled: Optional[pulumi.Input[_builtins.bool]] = None,
                 control_plane: Optional[pulumi.Input[Union['LkeClusterControlPlaneArgs', 'LkeClusterControlPlaneArgsDict']]] = None,
                 external_pool_tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 k8s_version: Optional[pulumi.Input[_builtins.str]] = None,
                 label: Optional[pulumi.Input[_builtins.str]] = None,
                 pools: Optional[pulumi.Input[Sequence[pulumi.Input[Union['LkeClusterPoolArgs', 'LkeClusterPoolArgsDict']]]]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 stack_type: Optional[pulumi.Input[_builtins.str]] = None,
                 subnet_id: Optional[pulumi.Input[_builtins.int]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 tier: Optional[pulumi.Input[_builtins.str]] = None,
                 vpc_id: Optional[pulumi.Input[_builtins.int]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = LkeClusterArgs.__new__(LkeClusterArgs)

            __props__.__dict__["apl_enabled"] = apl_enabled
            __props__.__dict__["control_plane"] = control_plane
            __props__.__dict__["external_pool_tags"] = external_pool_tags
            if k8s_version is None and not opts.urn:
                raise TypeError("Missing required property 'k8s_version'")
            __props__.__dict__["k8s_version"] = k8s_version
            if label is None and not opts.urn:
                raise TypeError("Missing required property 'label'")
            __props__.__dict__["label"] = label
            if pools is None and not opts.urn:
                raise TypeError("Missing required property 'pools'")
            __props__.__dict__["pools"] = pools
            if region is None and not opts.urn:
                raise TypeError("Missing required property 'region'")
            __props__.__dict__["region"] = region
            __props__.__dict__["stack_type"] = stack_type
            __props__.__dict__["subnet_id"] = subnet_id
            __props__.__dict__["tags"] = tags
            __props__.__dict__["tier"] = tier
            __props__.__dict__["vpc_id"] = vpc_id
            __props__.__dict__["api_endpoints"] = None
            __props__.__dict__["dashboard_url"] = None
            __props__.__dict__["kubeconfig"] = None
            __props__.__dict__["status"] = None
        secret_opts = pulumi.ResourceOptions(additional_secret_outputs=["kubeconfig"])
        opts = pulumi.ResourceOptions.merge(opts, secret_opts)
        super(LkeCluster, __self__).__init__(
            'linode:index/lkeCluster:LkeCluster',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            api_endpoints: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            apl_enabled: Optional[pulumi.Input[_builtins.bool]] = None,
            control_plane: Optional[pulumi.Input[Union['LkeClusterControlPlaneArgs', 'LkeClusterControlPlaneArgsDict']]] = None,
            dashboard_url: Optional[pulumi.Input[_builtins.str]] = None,
            external_pool_tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            k8s_version: Optional[pulumi.Input[_builtins.str]] = None,
            kubeconfig: Optional[pulumi.Input[_builtins.str]] = None,
            label: Optional[pulumi.Input[_builtins.str]] = None,
            pools: Optional[pulumi.Input[Sequence[pulumi.Input[Union['LkeClusterPoolArgs', 'LkeClusterPoolArgsDict']]]]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None,
            stack_type: Optional[pulumi.Input[_builtins.str]] = None,
            status: Optional[pulumi.Input[_builtins.str]] = None,
            subnet_id: Optional[pulumi.Input[_builtins.int]] = None,
            tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            tier: Optional[pulumi.Input[_builtins.str]] = None,
            vpc_id: Optional[pulumi.Input[_builtins.int]] = None) -> 'LkeCluster':
        """
        Get an existing LkeCluster resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] api_endpoints: The endpoints for the Kubernetes API server.
        :param pulumi.Input[_builtins.bool] apl_enabled: Enables the App Platform Layer
        :param pulumi.Input[Union['LkeClusterControlPlaneArgs', 'LkeClusterControlPlaneArgsDict']] control_plane: Defines settings for the Kubernetes Control Plane.
        :param pulumi.Input[_builtins.str] dashboard_url: The Kubernetes Dashboard access URL for this cluster. LKE Enterprise does not have a dashboard URL.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] external_pool_tags: A set of node pool tags to ignore when planning and applying this cluster. This prevents externally managed node pools from being deleted or unintentionally updated on subsequent applies. See Externally Managed Node Pools for more details.
        :param pulumi.Input[_builtins.str] k8s_version: The desired Kubernetes version for this Kubernetes cluster in the format of `major.minor` (e.g. `1.21`), and the latest supported patch version will be deployed.
        :param pulumi.Input[_builtins.str] kubeconfig: The base64 encoded kubeconfig for the Kubernetes cluster.
        :param pulumi.Input[_builtins.str] label: This Kubernetes cluster's unique label.
        :param pulumi.Input[Sequence[pulumi.Input[Union['LkeClusterPoolArgs', 'LkeClusterPoolArgsDict']]]] pools: Additional nested attributes:
        :param pulumi.Input[_builtins.str] region: This Kubernetes cluster's location.
               
               * `pool` - (Required) The Node Pool specifications for the Kubernetes cluster. At least one Node Pool is required.
               
               * `control_plane` (Optional) Defines settings for the Kubernetes Control Plane.
        :param pulumi.Input[_builtins.str] stack_type: The networking stack type of the Kubernetes cluster.
        :param pulumi.Input[_builtins.str] status: The status of the node. (`ready`, `not_ready`)
        :param pulumi.Input[_builtins.int] subnet_id: The ID of the VPC subnet to use for the Kubernetes cluster. This subnet must be dual stack (IPv4 and IPv6 should both be enabled). (**Note: v4beta only and may not currently be available to all users.**)
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: An array of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        :param pulumi.Input[_builtins.str] tier: The desired Kubernetes tier. (**Note: v4beta only and may not currently be available to all users.**)
        :param pulumi.Input[_builtins.int] vpc_id: The ID of the VPC to use for the Kubernetes cluster.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _LkeClusterState.__new__(_LkeClusterState)

        __props__.__dict__["api_endpoints"] = api_endpoints
        __props__.__dict__["apl_enabled"] = apl_enabled
        __props__.__dict__["control_plane"] = control_plane
        __props__.__dict__["dashboard_url"] = dashboard_url
        __props__.__dict__["external_pool_tags"] = external_pool_tags
        __props__.__dict__["k8s_version"] = k8s_version
        __props__.__dict__["kubeconfig"] = kubeconfig
        __props__.__dict__["label"] = label
        __props__.__dict__["pools"] = pools
        __props__.__dict__["region"] = region
        __props__.__dict__["stack_type"] = stack_type
        __props__.__dict__["status"] = status
        __props__.__dict__["subnet_id"] = subnet_id
        __props__.__dict__["tags"] = tags
        __props__.__dict__["tier"] = tier
        __props__.__dict__["vpc_id"] = vpc_id
        return LkeCluster(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="apiEndpoints")
    def api_endpoints(self) -> pulumi.Output[Sequence[_builtins.str]]:
        """
        The endpoints for the Kubernetes API server.
        """
        return pulumi.get(self, "api_endpoints")

    @_builtins.property
    @pulumi.getter(name="aplEnabled")
    def apl_enabled(self) -> pulumi.Output[_builtins.bool]:
        """
        Enables the App Platform Layer
        """
        return pulumi.get(self, "apl_enabled")

    @_builtins.property
    @pulumi.getter(name="controlPlane")
    def control_plane(self) -> pulumi.Output['outputs.LkeClusterControlPlane']:
        """
        Defines settings for the Kubernetes Control Plane.
        """
        return pulumi.get(self, "control_plane")

    @_builtins.property
    @pulumi.getter(name="dashboardUrl")
    def dashboard_url(self) -> pulumi.Output[_builtins.str]:
        """
        The Kubernetes Dashboard access URL for this cluster. LKE Enterprise does not have a dashboard URL.
        """
        return pulumi.get(self, "dashboard_url")

    @_builtins.property
    @pulumi.getter(name="externalPoolTags")
    def external_pool_tags(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        A set of node pool tags to ignore when planning and applying this cluster. This prevents externally managed node pools from being deleted or unintentionally updated on subsequent applies. See Externally Managed Node Pools for more details.
        """
        return pulumi.get(self, "external_pool_tags")

    @_builtins.property
    @pulumi.getter(name="k8sVersion")
    def k8s_version(self) -> pulumi.Output[_builtins.str]:
        """
        The desired Kubernetes version for this Kubernetes cluster in the format of `major.minor` (e.g. `1.21`), and the latest supported patch version will be deployed.
        """
        return pulumi.get(self, "k8s_version")

    @_builtins.property
    @pulumi.getter
    def kubeconfig(self) -> pulumi.Output[_builtins.str]:
        """
        The base64 encoded kubeconfig for the Kubernetes cluster.
        """
        return pulumi.get(self, "kubeconfig")

    @_builtins.property
    @pulumi.getter
    def label(self) -> pulumi.Output[_builtins.str]:
        """
        This Kubernetes cluster's unique label.
        """
        return pulumi.get(self, "label")

    @_builtins.property
    @pulumi.getter
    def pools(self) -> pulumi.Output[Sequence['outputs.LkeClusterPool']]:
        """
        Additional nested attributes:
        """
        return pulumi.get(self, "pools")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        This Kubernetes cluster's location.

        * `pool` - (Required) The Node Pool specifications for the Kubernetes cluster. At least one Node Pool is required.

        * `control_plane` (Optional) Defines settings for the Kubernetes Control Plane.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter(name="stackType")
    def stack_type(self) -> pulumi.Output[_builtins.str]:
        """
        The networking stack type of the Kubernetes cluster.
        """
        return pulumi.get(self, "stack_type")

    @_builtins.property
    @pulumi.getter
    def status(self) -> pulumi.Output[_builtins.str]:
        """
        The status of the node. (`ready`, `not_ready`)
        """
        return pulumi.get(self, "status")

    @_builtins.property
    @pulumi.getter(name="subnetId")
    def subnet_id(self) -> pulumi.Output[_builtins.int]:
        """
        The ID of the VPC subnet to use for the Kubernetes cluster. This subnet must be dual stack (IPv4 and IPv6 should both be enabled). (**Note: v4beta only and may not currently be available to all users.**)
        """
        return pulumi.get(self, "subnet_id")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Sequence[_builtins.str]]:
        """
        An array of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        """
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter
    def tier(self) -> pulumi.Output[_builtins.str]:
        """
        The desired Kubernetes tier. (**Note: v4beta only and may not currently be available to all users.**)
        """
        return pulumi.get(self, "tier")

    @_builtins.property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> pulumi.Output[_builtins.int]:
        """
        The ID of the VPC to use for the Kubernetes cluster.
        """
        return pulumi.get(self, "vpc_id")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetDomainsResult',
    'AwaitableGetDomainsResult',
    'get_domains',
    'get_domains_output',
]

@pulumi.output_type
class GetDomainsResult:
    """
    A collection of values returned by getDomains.
    """
    def __init__(__self__, domains=None, filters=None, id=None, order=None, order_by=None):
        if domains and not isinstance(domains, list):
            raise TypeError("Expected argument 'domains' to be a list")
        pulumi.set(__self__, "domains", domains)
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if order and not isinstance(order, str):
            raise TypeError("Expected argument 'order' to be a str")
        pulumi.set(__self__, "order", order)
        if order_by and not isinstance(order_by, str):
            raise TypeError("Expected argument 'order_by' to be a str")
        pulumi.set(__self__, "order_by", order_by)

    @_builtins.property
    @pulumi.getter
    def domains(self) -> Optional[Sequence['outputs.GetDomainsDomainResult']]:
        return pulumi.get(self, "domains")

    @_builtins.property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetDomainsFilterResult']]:
        return pulumi.get(self, "filters")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The unique ID of this Domain.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def order(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "order")

    @_builtins.property
    @pulumi.getter(name="orderBy")
    def order_by(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "order_by")


class AwaitableGetDomainsResult(GetDomainsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetDomainsResult(
            domains=self.domains,
            filters=self.filters,
            id=self.id,
            order=self.order,
            order_by=self.order_by)


def get_domains(domains: Optional[Sequence[Union['GetDomainsDomainArgs', 'GetDomainsDomainArgsDict']]] = None,
                filters: Optional[Sequence[Union['GetDomainsFilterArgs', 'GetDomainsFilterArgsDict']]] = None,
                order: Optional[_builtins.str] = None,
                order_by: Optional[_builtins.str] = None,
                opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetDomainsResult:
    """
    Provides information about Linode Domains that match a set of filters.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-domains).

    ## Example Usage

    Get information about all Linode Cloud Domains with a specific tag:

    ```python
    import pulumi
    import pulumi_linode as linode

    specific = linode.get_domains(filters=[{
        "name": "tags",
        "values": ["test-tag"],
    }])
    pulumi.export("domain", specific.domains[0].domain)
    ```

    ## Filterable Fields

    * `group`

    * `tags`

    * `domain`

    * `type`

    * `status`

    * `description`

    * `master_ips`

    * `axfr_ips`

    * `ttl_sec`

    * `retry_sec`

    * `expire_sec`

    * `refresh_sec`

    * `soa_email`


    :param _builtins.str order: The order in which results should be returned. (`asc`, `desc`; default `asc`)
    :param _builtins.str order_by: The attribute to order the results by. See the Filterable Fields section for a list of valid fields.
    """
    __args__ = dict()
    __args__['domains'] = domains
    __args__['filters'] = filters
    __args__['order'] = order
    __args__['orderBy'] = order_by
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('linode:index/getDomains:getDomains', __args__, opts=opts, typ=GetDomainsResult).value

    return AwaitableGetDomainsResult(
        domains=pulumi.get(__ret__, 'domains'),
        filters=pulumi.get(__ret__, 'filters'),
        id=pulumi.get(__ret__, 'id'),
        order=pulumi.get(__ret__, 'order'),
        order_by=pulumi.get(__ret__, 'order_by'))
def get_domains_output(domains: Optional[pulumi.Input[Optional[Sequence[Union['GetDomainsDomainArgs', 'GetDomainsDomainArgsDict']]]]] = None,
                       filters: Optional[pulumi.Input[Optional[Sequence[Union['GetDomainsFilterArgs', 'GetDomainsFilterArgsDict']]]]] = None,
                       order: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                       order_by: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                       opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetDomainsResult]:
    """
    Provides information about Linode Domains that match a set of filters.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-domains).

    ## Example Usage

    Get information about all Linode Cloud Domains with a specific tag:

    ```python
    import pulumi
    import pulumi_linode as linode

    specific = linode.get_domains(filters=[{
        "name": "tags",
        "values": ["test-tag"],
    }])
    pulumi.export("domain", specific.domains[0].domain)
    ```

    ## Filterable Fields

    * `group`

    * `tags`

    * `domain`

    * `type`

    * `status`

    * `description`

    * `master_ips`

    * `axfr_ips`

    * `ttl_sec`

    * `retry_sec`

    * `expire_sec`

    * `refresh_sec`

    * `soa_email`


    :param _builtins.str order: The order in which results should be returned. (`asc`, `desc`; default `asc`)
    :param _builtins.str order_by: The attribute to order the results by. See the Filterable Fields section for a list of valid fields.
    """
    __args__ = dict()
    __args__['domains'] = domains
    __args__['filters'] = filters
    __args__['order'] = order
    __args__['orderBy'] = order_by
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('linode:index/getDomains:getDomains', __args__, opts=opts, typ=GetDomainsResult)
    return __ret__.apply(lambda __response__: GetDomainsResult(
        domains=pulumi.get(__response__, 'domains'),
        filters=pulumi.get(__response__, 'filters'),
        id=pulumi.get(__response__, 'id'),
        order=pulumi.get(__response__, 'order'),
        order_by=pulumi.get(__response__, 'order_by')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['NetworkingIpAssignmentArgs', 'NetworkingIpAssignment']

@pulumi.input_type
class NetworkingIpAssignmentArgs:
    def __init__(__self__, *,
                 region: pulumi.Input[_builtins.str],
                 assignments: Optional[pulumi.Input[Sequence[pulumi.Input['NetworkingIpAssignmentAssignmentArgs']]]] = None):
        """
        The set of arguments for constructing a NetworkingIpAssignment resource.
        :param pulumi.Input[_builtins.str] region: The region where the IP addresses will be assigned.
        :param pulumi.Input[Sequence[pulumi.Input['NetworkingIpAssignmentAssignmentArgs']]] assignments: A list of IP/Linode assignments to apply.
        """
        pulumi.set(__self__, "region", region)
        if assignments is not None:
            pulumi.set(__self__, "assignments", assignments)

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Input[_builtins.str]:
        """
        The region where the IP addresses will be assigned.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def assignments(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['NetworkingIpAssignmentAssignmentArgs']]]]:
        """
        A list of IP/Linode assignments to apply.
        """
        return pulumi.get(self, "assignments")

    @assignments.setter
    def assignments(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['NetworkingIpAssignmentAssignmentArgs']]]]):
        pulumi.set(self, "assignments", value)


@pulumi.input_type
class _NetworkingIpAssignmentState:
    def __init__(__self__, *,
                 assignments: Optional[pulumi.Input[Sequence[pulumi.Input['NetworkingIpAssignmentAssignmentArgs']]]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering NetworkingIpAssignment resources.
        :param pulumi.Input[Sequence[pulumi.Input['NetworkingIpAssignmentAssignmentArgs']]] assignments: A list of IP/Linode assignments to apply.
        :param pulumi.Input[_builtins.str] region: The region where the IP addresses will be assigned.
        """
        if assignments is not None:
            pulumi.set(__self__, "assignments", assignments)
        if region is not None:
            pulumi.set(__self__, "region", region)

    @_builtins.property
    @pulumi.getter
    def assignments(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['NetworkingIpAssignmentAssignmentArgs']]]]:
        """
        A list of IP/Linode assignments to apply.
        """
        return pulumi.get(self, "assignments")

    @assignments.setter
    def assignments(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['NetworkingIpAssignmentAssignmentArgs']]]]):
        pulumi.set(self, "assignments", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region where the IP addresses will be assigned.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)


@pulumi.type_token("linode:index/networkingIpAssignment:NetworkingIpAssignment")
class NetworkingIpAssignment(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 assignments: Optional[pulumi.Input[Sequence[pulumi.Input[Union['NetworkingIpAssignmentAssignmentArgs', 'NetworkingIpAssignmentAssignmentArgsDict']]]]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Manages the assignment of multiple IPv4 addresses and/or IPv6 ranges to multiple Linodes in a specified region.

        For more information, see the corresponding [API documentation](https://techdocs.akamai.com/linode-api/reference/post-assign-ips).

        ## Example Usage

        ```python
        import pulumi
        import pulumi_linode as linode

        foobar = linode.NetworkingIpAssignment("foobar",
            region="us-mia",
            assignments=[
                {
                    "address": reserved_ip1["address"],
                    "linode_id": terraform_web1["id"],
                },
                {
                    "address": reserved_ip2["address"],
                    "linode_id": terraform_web2["id"],
                },
            ])
        ```

        ## Import

        Network IP assignments cannot be imported.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['NetworkingIpAssignmentAssignmentArgs', 'NetworkingIpAssignmentAssignmentArgsDict']]]] assignments: A list of IP/Linode assignments to apply.
        :param pulumi.Input[_builtins.str] region: The region where the IP addresses will be assigned.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: NetworkingIpAssignmentArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages the assignment of multiple IPv4 addresses and/or IPv6 ranges to multiple Linodes in a specified region.

        For more information, see the corresponding [API documentation](https://techdocs.akamai.com/linode-api/reference/post-assign-ips).

        ## Example Usage

        ```python
        import pulumi
        import pulumi_linode as linode

        foobar = linode.NetworkingIpAssignment("foobar",
            region="us-mia",
            assignments=[
                {
                    "address": reserved_ip1["address"],
                    "linode_id": terraform_web1["id"],
                },
                {
                    "address": reserved_ip2["address"],
                    "linode_id": terraform_web2["id"],
                },
            ])
        ```

        ## Import

        Network IP assignments cannot be imported.

        :param str resource_name: The name of the resource.
        :param NetworkingIpAssignmentArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(NetworkingIpAssignmentArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 assignments: Optional[pulumi.Input[Sequence[pulumi.Input[Union['NetworkingIpAssignmentAssignmentArgs', 'NetworkingIpAssignmentAssignmentArgsDict']]]]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = NetworkingIpAssignmentArgs.__new__(NetworkingIpAssignmentArgs)

            __props__.__dict__["assignments"] = assignments
            if region is None and not opts.urn:
                raise TypeError("Missing required property 'region'")
            __props__.__dict__["region"] = region
        super(NetworkingIpAssignment, __self__).__init__(
            'linode:index/networkingIpAssignment:NetworkingIpAssignment',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            assignments: Optional[pulumi.Input[Sequence[pulumi.Input[Union['NetworkingIpAssignmentAssignmentArgs', 'NetworkingIpAssignmentAssignmentArgsDict']]]]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None) -> 'NetworkingIpAssignment':
        """
        Get an existing NetworkingIpAssignment resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['NetworkingIpAssignmentAssignmentArgs', 'NetworkingIpAssignmentAssignmentArgsDict']]]] assignments: A list of IP/Linode assignments to apply.
        :param pulumi.Input[_builtins.str] region: The region where the IP addresses will be assigned.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _NetworkingIpAssignmentState.__new__(_NetworkingIpAssignmentState)

        __props__.__dict__["assignments"] = assignments
        __props__.__dict__["region"] = region
        return NetworkingIpAssignment(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter
    def assignments(self) -> pulumi.Output[Optional[Sequence['outputs.NetworkingIpAssignmentAssignment']]]:
        """
        A list of IP/Linode assignments to apply.
        """
        return pulumi.get(self, "assignments")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        The region where the IP addresses will be assigned.
        """
        return pulumi.get(self, "region")

